package de.kleopatra.forms.description;

import java.beans.PropertyChangeListener;

/** Description of constraint/component pair.
 *  <p>
 *  The constraint part has notion of location/extension/alignment.
 *  The component part has descriptive hints for later component creation:
 *    componentID, componentClassName
 *
 *  @version $Revision: 1.1.1.1 $ $Date: 2004/05/17 15:03:43 $
 *  @author (C) 2003 Jeanette Winzenburg
 *
 */
public interface Constraint {

	public void addPropertyChangeListener(PropertyChangeListener l);

	public void removePropertyChangeListener(PropertyChangeListener l);

	public Object getComponentID();

	public void setComponentID(Object componentID);

	public String getComponentClassName();

	public void setComponentClassName(String className);

	/** a String representation capable of instantiating a valid
	 * constraint.
	 * <p>
	 * Note: this method is meant mainly for handling persistence and
	 * will not fire any event. Hmmm...
	 */
	public String getEncodedString();

	public void setEncodedString(String encodedString);

	public Object getHAlignment();

	public void setHAlignment(Object alignment);

	public Object getVAlignment();

	public void setVAlignment(Object alignment);

	public int getFirstRow();

	public int getFirstColumn();

	public int getLastRow();

	public int getLastColumn();

	public int getHeight();

	public int getWidth();

	public boolean isAnchor(int row, int column);

	public boolean isInside(int row, int column);

//  public boolean isInsideRowRange(int row);
//  public boolean isInsideColumnRange(int column);
	public String getDescription();

// THINK: these are for use in LayoutBuilder only.
// other Clients are not allowed to change the location/extension of
// constraints that are already inserted into a layoutBuilder
// how to hide??
	public void extend(int rowDelta, int columnDelta);

	public void moveTo(int row, int column);
}
