package de.kleopatra.forms.description;

import java.util.logging.Logger;

import de.kleopatra.forms.description.jgoodies.JGoodiesLayoutFactory;
import de.kleopatra.forms.description.jgoodies.JGoodiesLayoutTranslator;
import de.kleopatra.forms.view.builder.MappingManager;


/** factory to create the factories for creation of LayoutElements.
 *
 *
 *  @version $Revision: 1.1.1.1 $ $Date: 2004/06/04 10:03:48 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public class LayoutBuilderFactory {

  private static final Logger LOGGER = Logger.getLogger("LayoutBuilderFactory");
	private static LayoutFactory layoutFactory;
	private static LayoutTranslator translator;
	public static LayoutFactory getLayoutFactory() {
		if (layoutFactory == null) {
			layoutFactory = createLayoutFactory();
	//		initStyleMap(layoutFactory);
		}
		return layoutFactory;
	}

	private static void initStyleMap(LayoutFactory factory) {
		Class mappingClass = MappingManager.getInstance().getStyleMapClass();
		try {
			StyleMap map = (StyleMap) mappingClass.newInstance();
			factory.setStyleMap(map);
			LOGGER.info("setting StyleMap: "  + mappingClass);
		} catch (Exception ex) {
			LOGGER.info("could not create StyleMap: "  + mappingClass + ". Will use default");
		}
  }

  public static LayoutTranslator getLayoutTranslator() {
		if (translator == null) {
			translator = createTranslator();
		}
		return translator;
	}

	//---------------------------factory methods

	/** encapsulate creation of layoutFactory per project.
	 *      this is the method to change if subclasses are introduced
	 **/
	private static synchronized LayoutFactory createLayoutFactory() {
	  LayoutFactory factory = new JGoodiesLayoutFactory();
	  initStyleMap(factory);
		return factory;
	}

	private static LayoutTranslator createTranslator() {
		try {
			return new JGoodiesLayoutTranslator();

//      (LayoutTranslator) ResourceManager.getInstance().getClass(
//        "de.kleopatra.forms.description.jgoodies.JGoodiesLayoutTranslator")
//          .newInstance();
		} catch (Exception ex) {
			throw new RuntimeException("got a problem in LayoutBuilderFactory: "
					+ ex);
		}
	}
}
