package de.kleopatra.forms.description;

import java.awt.Component;
import java.awt.LayoutManager;
import java.util.List;

import javax.swing.border.Border;

/** Responsible to translate description elements
 *  to layout elements as understood by a specific LayoutManager and
 *  vice versa.
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public interface LayoutTranslator {

//---------------------------translate generic elements into specific

	/** returns a constraint object that's understood by the created layoutManager.
	 */
	public Object createConstraintSpec(Constraint wrapper);

	/** returns a layoutManager with the rows, columns.
	 *  rows/columns are lists of Row/Column objects
	 */
	public LayoutManager createLayoutManager(List rows, List columns);

	/** returns a painter that can visualize layoutInfo.
	 *
	 */
	public LayoutDebugPainter createLayoutDebugPainter();

	public Border createBorder(LayoutBorder border);

//---------------------------translate specific elements into generic
	public boolean isCompatible(LayoutManager layoutManager);

	public Constraint createConstraint(Object constraintSpec);

	public Row createRow(Object rowSpec);

	public Column createColumn(Object columnSpec);

	/** returns a Constraint for the given Component.
	 */
	public Constraint createConstraint(LayoutManager layoutManager,
			Component comp);

	/** returns a list of Row descriptions from the manager's rows.
	 */
	public List createRows(LayoutManager layoutManager);

	/** return a list of Column descriptions from the manager's columns.
	 */
	public List createColumns(LayoutManager layoutManager);
}
