/*
 * Created on 14.05.2004
 *
 */
package de.kleopatra.forms.description;

import java.util.List;

/**
 * Object that is mutable by Style.
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/05/17 15:03:43 $
 */
public interface Stylable {

	/** Returns a list of all registered styles.<p>
	 *  Items of the List are guaranteed to be valid
	 *  entries in setType.
	 *
	 * @return
	 */
	public List getStyles();

	/** returns the current style.
	 *
	 * @return
	 */
	public Style getStyle();

	/** get/setType for use in (visual) builders. <p>
	 *  actually getType returns getStyle - but for
	 *  convenient binding the return type has to be
	 *  the same as the parameter type in the setter.
	 *
	 * @return
	 */
	public Object getType();

	/**
	 * A lenient setter for Style. <p>
	 * It's up the the concrete implemantation to define
	 * valid input types and formats. Minimally it must guarantee
	 * to accept all objects returned in the getStyles() and the
	 * output of getValidEncoding().
	 *
	 * @param style
	 */
	public void setType(Object style);

	/** legacy (?)
	 *
	 * @return
	 */
	public String getValidEncoding();
}
