/*
 * Created on 10.05.2004
 *
 */
package de.kleopatra.forms.description;

import java.util.List;

/**
 *  Handler (Factory?) that keeps track of available Styles.<p>
 *
 *  It guarantees to create and return Styles that
 *  can be set to Stylables of the same context.  
 *  A Style as such is immutable but can be 
 *  modified by adding/removing/renaming in the StyleMap
 *  if the map marks it as configurable. <p>
 *
 *  Implementations should make it impossible to
 *  modify the default types (as given by
 *  the Forms LayoutStyle/FormFactory, f.i.). If clients
 *  want to change the base types they need to provide a
 *  custom LayoutStyle. <p>
 *
 *	
 *
 *  Known issues:
 *  <ul>
 *  <li> The style map used in the builder (for building itself)
 *  is the same as the custom map, so the builder might look funny
 *  if a very wild custom LayoutStyle is used.
 *  <li> persisting a custom style may produce a double entry for
 *  property "logicalType" in the output stream. Though it's
 *  annoying it's not disturbing functionality.
 *  </ul>
 *
 * NOTE: this api is unstable ... need some experience about what's
 * really needed. 
 * 
 * @author (C) 2004, Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/07/01 12:39:59 $
 */
public interface StyleMap {

//------------------ accessing available logical types
	public abstract List getRowContentStyles();

	public abstract List getRowGapStyles();

	public abstract Style getRowGlueStyle();

	public abstract List getColumnContentStyles();

	public abstract List getColumnGapStyles();

	public abstract Style getColumnGlueStyle();

	public abstract List getBorderStyles();

//------------------------- accessing Style by typeKey
	
  public abstract Style getColumnContentStyle(Object typeKey);
  public abstract Style getColumnGapStyle(Object typeKey);
  public abstract Style getRowContentStyle(Object typeKey);
  public abstract Style getRowGapStyle(Object typeKey);
  public abstract Style getBorderStyle(Object typeKey);
  

//---------------------- modifying Styles  

	public boolean isConfigurableRowStyle(Object typeKey);
	public boolean isConfigurableColumnStyle(Object typeKey);
	public boolean isConfigurableBorderStyle(Object typeKey);
	
  
//------------------------ adding types
	public abstract void renameColumnContentStyle(Object oldTypeKey, Object newTypeKey);
  public abstract void renameColumnGapStyle(Object oldTypeKey, Object newTypeKey);
  public abstract void renameBorderStyle(Object oldTypeKey, Object newTypeKey);

  public abstract void renameRowContentStyle(Object oldTypeKey, Object newTypeKey);

  public abstract void renameRowGapStyle(Object oldTypeKey, Object newTypeKey);


	public abstract void addRowContentStyle(Object typeKey,
			String encodedString, String description);

	public abstract void addRowGapStyle(Object typeKey, String encodedString,
			String description);

	public abstract void addColumnContentStyle(Object typeKey,
			String encodedString, String description);

	public abstract void addColumnGapStyle(Object typeKey, String encodedString,
			String description);

	public abstract void addBorderStyle(Object typeKey, String encoding,
			String description);

//----------------------- removing types
	public abstract void removeRowContentStyle(Object typeKey);

	public abstract void removeRowGapStyle(Object typeKey);

	public abstract void removeColumnContentStyle(Object typeKey);

	public abstract void removeColumnGapStyle(Object typeKey);

	public abstract void removeBorderStyle(Object typeKey);


}
