/*
 * Created on 22.09.2003
 *
 */
package de.kleopatra.forms.description.jgoodies;

import java.beans.*;
import java.util.Iterator;

import de.kleopatra.forms.description.Stylable;
import de.kleopatra.forms.description.Style;

/** Abstract Stylable meant for persistence.
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/06/10 11:41:37 $
 */
public abstract class AbstractStylable implements Stylable {

	protected Style styleDelegate;

//--------------------------"mutable" style for persistence
	public String getEncodedString() {
		if (!isPredefinedStyle(getLogicalType())) {
			return getValidEncoding();
		}
		return null;    //getValidEncoding();
	}

	public Object getLogicalType() {
		return getStyle().getLogicalType();
	}

	public String getDescription() {
		return getStyle().getDescription();
	}

	public void setEncodedString(String border) {
		setStyle(createStyle(border));
	}

	public void setLogicalType(Object typeKey) {
		Style type = findStyleByKey(typeKey);
		if (type == null) {
			type = createStyle(getStyle(), typeKey);
		}
		setStyle(type);
	}

	public static PersistenceDelegate createPersistenceDelegate() {
		DefaultPersistenceDelegate delegate = new DefaultPersistenceDelegate() {

			protected void initialize(Class clazz, Object oldInstance,
					Object newInstance, Encoder out) {
				super.initialize(clazz, oldInstance, newInstance, out);
				AbstractStylable line = (AbstractStylable) oldInstance;
				if ((line.getEncodedString() != null) //) {
				  // preventing double write-out of logical type
				  // if we don't know the style:
				  // next line is false if we don't know the style
				  // in which case super has already written it.
				  // if we know the style super will not have written it
				  // we have to make sure that decoders not knowing
				  // the style will get a style info.
					&&  (line.findStyleByKey(line.getLogicalType()) != null)) {
					out.writeStatement(new Statement(oldInstance, "setLogicalType",
							new Object[]{ line.getLogicalType() }));
				}
			}
		};
		return delegate;
	}
	
//----------------------- Stylable
	public Object getType() {
		return getStyle();
	}

	public void setType(Object desc) {
		if (checkStyle(desc)) {
			setStyle((Style) desc);
		} else {
			String encoded = FormUtils.stripEncoding(String.valueOf(desc));
			Object comparing = createComparable(encoded);
			String comparableEncoding = FormUtils.getEncodedString(comparing);
			setEncodedString(comparableEncoding);
		}
	}

	public Style getStyle() {
		if (styleDelegate == null) {
			return getDefaultStyle();
		}
		return styleDelegate;
	}

	public String getValidEncoding() {
		return getStyle().getEncodedString();
	}

//------------------------------- abstract 
	protected abstract boolean isPredefinedStyle(Object typeKey);

	protected abstract Style createCustomStyle(String encodedString);

	protected abstract Style createStyle(Style current, Object newTypeKey);

	protected abstract Style getDefaultStyle();

	protected abstract boolean checkStyle(Object style);

	protected abstract Object createComparable(String encoding);

//----------------------------- helpers for use in descendants

	/** pre: checkStyle.
	 *
	 */
	protected void setStyle(Style style) {
		styleDelegate = style;
	}

	protected Style findStyleByKey(Object typeKey) {
		for (Iterator iterator = getStyles().iterator(); iterator.hasNext(); ) {
			Style element = (Style) iterator.next();
			if (typeKey.equals(element.getLogicalType())) {
				return element;
			}
		}
		return null;
	}

	protected Style findStyleByEncoding(String encodedString) {
		if ((encodedString == null) || (encodedString.length() == 0)) {
			return getDefaultStyle();
		}
		return FormUtils.findStyleByEncoding(getStyles(), encodedString);
	}

	protected Style createStyle(String encodedString) {
		Object knownType = findStyleByEncoding(encodedString);
		if (checkStyle(knownType)) {
			return (Style) knownType;
		}
		return createCustomStyle(encodedString);
	}
}

