/*
 * Created on 05.09.2003
 *
 */
package de.kleopatra.forms.description.jgoodies;

import java.util.Iterator;
import java.util.List;

import javax.swing.border.Border;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.RowSpec;

import de.kleopatra.forms.description.Style;

/**
 *  collection of static utility methods across formspecs et al.
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/05/17 15:03:43 $
 */
class FormUtils {

	public static final String OPEN_DEL = "[";
	public static final String CLOSE_DEL = "]";

	/** processes possibleEncoding by removing
	 *  all char outside the OPEN/CLOSE_DEL and any
	 *  spaces inside.
	 *
	 * @param possibleEncoding
	 * @return the processed encoding or null if it has been null.
	 */
	public static String stripEncoding(String possibleEncoding) {
		if (possibleEncoding == null) {
			return null;
		}
		int open = possibleEncoding.indexOf(FormUtils.OPEN_DEL);
		if (open >= 0) {
			possibleEncoding = possibleEncoding.substring(open + 1);
		}
		int close = possibleEncoding.indexOf(FormUtils.CLOSE_DEL);
		if (close >= 0) {
			possibleEncoding = possibleEncoding.substring(0, close);
		}
		possibleEncoding = possibleEncoding.replaceAll(" ", "");
		return possibleEncoding;
	}

//----------------------------Style

	/** search a list of Styles by encodedString. <p>
	 *  Note: the given encoding has to be a valid encoding
	 *  or a valid encoding inside square brackets.
	 *
	 */
	public static Style findStyleByEncoding(List list, String encoded) {
		encoded = stripEncoding(encoded);
		for (Iterator iter = list.iterator(); iter.hasNext(); ) {
			Style element = (Style) iter.next();
			if (encoded.equals(element.getEncodedString())) {
				return element;
			}
		}
		return null;
	}

	//---------------------------Spec

	/** returns a valid encoding from Row/ColumnSpec and Border or
	 *  an empty String.
	 */
	public static String getEncodedString(Object spec) {
		if (spec instanceof RowSpec) {
			return getEncodedString((RowSpec) spec);
		}
		if (spec instanceof ColumnSpec) {
			return getEncodedString((ColumnSpec) spec);
		}
		if (spec instanceof Borders.EmptyBorder) {
			return getEncodedString((Borders.EmptyBorder) spec);
		}
		return "";
	}

	//---------------------------Row
	public static String getEncodedString(RowSpec spec) {
		if (spec == null) {
			return "";
		}
		String encoded = spec.toShortString();
		return encoded.replaceAll("Y", "");
	}

	public static boolean areEqual(RowSpec first, RowSpec second) {
		if ((first == null) || (second == null)) {
			return false;
		}
		return getEncodedString(first).equals(getEncodedString(second));
	}

	public static String getSpecDescription(Object spec) {
		return OPEN_DEL + getEncodedString(spec) + CLOSE_DEL;
	}

	public static String getLongSpecDescription(Object spec) {
		return OPEN_DEL + ((spec == null) ? "" : spec.toString()) + CLOSE_DEL;
	}

	//---------------------------Column
	public static String getEncodedString(ColumnSpec spec) {
		if (spec == null) {
			return "";
		}
		String encoded = spec.toShortString();
		return encoded.replaceAll("X", "");
	}

	public static boolean areEqual(ColumnSpec first, ColumnSpec second) {
		if ((first == null) || (second == null)) {
			return false;
		}
		return getEncodedString(first).equals(getEncodedString(second));
	}

//  public static LineType findLineTypeByEncoding(List list, String encoded) {
//    for (Iterator iter = list.iterator(); iter.hasNext(); ) {
//      LineType element = (LineType) iter.next();
//      if (encoded.equals(FormUtils.getEncodedString(element.getSpec()))) {
//        return element;
//      }
//    }
//    return null;
//  }
//----------------------------borders
	public static String getBorderDescription(String encoded) {
		return OPEN_DEL + encoded + CLOSE_DEL;
	}

	public static String getEncodedString(Border border) {

		// PENDING: null or empty String? 
		// empty String is consistent with behaviour of 
		// getEncodedString(Object)
		if (border == null) {
			return "";
		}
		if (!(border instanceof Borders.EmptyBorder)) {
			throw new IllegalArgumentException(
				"FormUtils can only handle Borders.EmptyBorder");
		}
		Borders.EmptyBorder b = (Borders.EmptyBorder) border;
		String result = b.top().toString() + "," + b.left().toString() + ","
											+ b.bottom().toString() + "," + b.right().toString();
		result = result.replaceAll("X", "");
		return result.replaceAll("Y", "");
	}

//  public static BorderType findBorderTypeByEncoding(List list,
//      String encoded) {
//    encoded = stripEncoding(encoded);
//    for (Iterator iter = list.iterator(); iter.hasNext(); ) {
//      BorderType element = (BorderType) iter.next();
//      if (encoded.equals(element.getValidEncoding())) {
//        return element;
//      }
//    }
//    return null;
//  }
}
