package de.kleopatra.forms.description.jgoodies;

import java.util.Arrays;
import java.util.List;

import com.jgoodies.forms.layout.ColumnSpec;

import de.kleopatra.forms.description.*;

/** Implementation of Column for JGoodies Forms.
 *
 * <p>
 * NOTE: this class is public and has public constructors
 * as an implementation side-effect, only. Don't instantiate/access
 * directly from code, use the factory methods instead.
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:20 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public final class JGoodiesColumn extends JGoodiesLine implements Column {

	private static ColumnStyle DEFAULT_SPEC_DESCRIPTION =
		new ColumnStyle(DEFAULT_CONTENT, new ColumnSpec("f:d:n"), "default");

//-------------------------------- constructors
	public JGoodiesColumn() {
		this(DEFAULT_SPEC_DESCRIPTION);
	}

/*  public JGoodiesColumn(ColumnSpec spec) {
		this.spec = spec;
	}
*/
	public JGoodiesColumn(ColumnStyle specDescription) {
		setStyle(specDescription);
	}

//------------------------ convenience 
	public ColumnSpec getColumnSpec() {
		return getColumnType().getColumnSpec();
	}

//--------------------------implement abstract JGoodiesLine methods
	protected List getContentTypes() {
		return LayoutBuilderFactory.getLayoutFactory().getColumnContentTypes();
	}

	protected List getGapTypes() {
		return LayoutBuilderFactory.getLayoutFactory().getColumnGapTypes();
	}

	protected Object getGlueType() {
		return LayoutBuilderFactory.getLayoutFactory().getColumnGlueType();
	}

//--------------------------- implement abstract AbstractPersistableStyle
	protected boolean isPredefinedStyle(Object typeKey) {
		return Arrays.asList(PREDEFINED_TYPES).contains(typeKey);
	}

	protected Style createStyle(Style current, Object newTypeKey) {
		ColumnStyle column = (ColumnStyle) current;
		return new ColumnStyle(newTypeKey, column.getColumnSpec(),
				String.valueOf(newTypeKey));
	}

	protected Style createCustomStyle(String encodedString) {
		return new ColumnStyle(CUSTOM, new ColumnSpec(encodedString), "custom");
	}

	protected Style getDefaultStyle() {
		return DEFAULT_SPEC_DESCRIPTION;
	}

	protected boolean checkStyle(Object style) {
		return style instanceof ColumnStyle;
	}

	protected Object createComparable(String encoding) {
		return new ColumnSpec(encoding);
	}

//-------------------------- convenience
	private ColumnStyle getColumnType() {
		return (ColumnStyle) getStyle();
	}
}

