package de.kleopatra.forms.description.jgoodies;

import java.beans.*;
import java.util.*;

import com.jgoodies.forms.layout.*;

import de.kleopatra.forms.description.*;
import de.kleopatra.persistence.Persist;

/** JGoodies Forms specific implementation of LayoutFactory.
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public final class JGoodiesLayoutFactory  // extends Model 
	implements LayoutFactory  {

	private CellConstraints.Alignment[] hAligns = { CellConstraints.DEFAULT,
																									CellConstraints.FILL,
																									CellConstraints.LEFT,
																									CellConstraints.CENTER,
																									CellConstraints.RIGHT };
	private String[] hAlignDescriptions = { "default", "fill", "left", "center",
																					"right" };
	private CellConstraints.Alignment[] vAligns = { CellConstraints.DEFAULT,
																									CellConstraints.FILL,
																									CellConstraints.TOP,
																									CellConstraints.CENTER,
																									CellConstraints.BOTTOM };
	private String[] vAlignDescriptions = { "default", "fill", "top", "center",
																					"bottom" };
	private List hAlignsList;
	private List vAlignsList;
	private StyleMap styleMap;
	
	private PropertyChangeSupport changeSupport;
	
	public JGoodiesLayoutFactory() {
		initPersistenceDelegates();
	}

	//---------------------------information methods
	public List getRowContentTypes() {
		return getStyleMap().getRowContentStyles();
	}

	public StyleMap getStyleMap() {
		if (styleMap == null) {
			styleMap = new JGoodiesStyleMap();
		}
		return styleMap;
	}

  public void setStyleMap(StyleMap map) {
    checkStyleMap(map);
    StyleMap oldMap = this.styleMap;
    styleMap = map;
    firePropertyChange("styleMap", oldMap, styleMap);
  }

  public void addPropertyChangeListener(PropertyChangeListener l) {
    if (changeSupport == null) {
      changeSupport = new PropertyChangeSupport(this);
    }
    changeSupport.addPropertyChangeListener(l);
    
  }


  public void removePropertyChangeListener(PropertyChangeListener l) {
    if (changeSupport == null) return;
    changeSupport.removePropertyChangeListener(l);
  }
  
  protected void firePropertyChange(String property, Object oldValue, Object newValue) {
    if (changeSupport == null) return;
    changeSupport.firePropertyChange(property, oldValue, newValue);
  }

  
	private void checkStyleMap(StyleMap map) {
    if (!(map.getRowGlueStyle() instanceof RowStyle)) {
      throw new IllegalArgumentException("StyleMap not compatible" + map);
    }
    
  }

  public List getRowGapTypes() {
		return getStyleMap().getRowGapStyles();
	}

	public Object getRowGlueType() {
		return getStyleMap().getRowGlueStyle();
	}

	public List getColumnContentTypes() {
		return getStyleMap().getColumnContentStyles();
	}

	public List getColumnGapTypes() {
		return getStyleMap().getColumnGapStyles();
	}

	public Object getColumnGlueType() {
		return getStyleMap().getColumnGlueStyle();
	}

	public List getBorderTypes() {
		return getStyleMap().getBorderStyles();
	}

	public List getConstraintHAlignmentTypes() {
		if (hAlignsList == null) {
			hAlignsList = Collections.unmodifiableList(Arrays.asList(hAligns));
		}
		return hAlignsList;
	}

	public List getConstraintVAlignmentTypes() {
		if (vAlignsList == null) {
			vAlignsList = Collections.unmodifiableList(Arrays.asList(vAligns));
		}
		return vAlignsList;
	}

//---------------------------factory methods
	public Row createRow() {
		return createJGoodiesRow((RowStyle) getRowContentTypes().get(0));
	}

	public Row createRow(String encodedString) {
		JGoodiesRow row = new JGoodiesRow();
		row.setEncodedString(encodedString);
		return row;
	}

	public Row createRow(Object contentType) {
		if (getRowContentTypes().contains(contentType)) {
			return createJGoodiesRow(((RowStyle) contentType));
		}
		return createRow(FormUtils.stripEncoding(String.valueOf(contentType)));
	}

	public Row createRowGap() {
		return createJGoodiesRow((RowStyle) getRowGapTypes().get(0));
	}

	public Row createRowGap(Object gapType) {
		if (getRowGapTypes().contains(gapType)) {
			return createJGoodiesRow(((RowStyle) gapType));
		}
		return createRow(FormUtils.stripEncoding(String.valueOf(gapType)));
	}

	public Row createRowGlue() {
		return createJGoodiesRow((RowStyle) getRowGlueType());
	}

	public Column createColumn() {
		return createJGoodiesColumn((ColumnStyle) getColumnContentTypes().get(0));
	}

	public Column createColumn(String encodedString) {
		JGoodiesColumn column = new JGoodiesColumn();
		column.setEncodedString(encodedString);
		return column;
	}

	public Column createColumn(Object contentType) {
		if (getColumnContentTypes().contains(contentType)) {
			return createJGoodiesColumn((ColumnStyle) contentType);
		}
		return createColumn(FormUtils.stripEncoding(String.valueOf(contentType)));
	}

	public Column createColumnGap() {
		return createJGoodiesColumn((ColumnStyle) getColumnGapTypes().get(0));
	}

	public Column createColumnGap(Object gapType) {
		if (getColumnGapTypes().contains(gapType)) {
			return createJGoodiesColumn((ColumnStyle) gapType);
		}
		return createColumn(FormUtils.stripEncoding(String.valueOf(gapType)));
	}

	public Column createColumnGlue() {
		return createJGoodiesColumn((ColumnStyle) getColumnGlueType());
	}

	public Constraint createConstraint() {
		return new JGoodiesConstraint();
	}

	public Constraint createConstraint(String encodedString) {
		return new JGoodiesConstraint(encodedString);
	}

	// PENDING: hacky shortcut, introduce LogicalBorderType!!
	public LayoutBorder createBorder() {
		return new JGoodiesBorder((BorderStyle) getStyleMap()
			.getBorderStyles().get(0));    //(BorderDescription) getLogicalTypeMap().getBorderTypes().get(0)); //createBorder("0dlu, 0dlu, 0dlu, 0dlu"); 
	}

	public LayoutBorder createBorder(String encodedString) {
		try {
			JGoodiesBorder border = new JGoodiesBorder();

			// to check if encodedString is valid
			border.setEncodedString(encodedString);
			return border;
		} catch (Exception e) {
			return null;
		}
	}

	public LayoutBorder createBorder(Object borderType) {
		if ((borderType == null) || (borderType.toString().length() == 0)) {
			return createBorder();
		}
		if (borderType instanceof BorderStyle) {
			return new JGoodiesBorder((BorderStyle) borderType);
		}
		return createBorder(borderType.toString());
	}

//---------------------------creating wrapper around JGoodies specs
	protected Row createJGoodiesRow(RowStyle spec) {
		return new JGoodiesRow(spec);
	}

	protected Column createJGoodiesColumn(ColumnStyle spec) {
		return new JGoodiesColumn(spec);
	}

	protected Constraint createJGoodiesConstraint(CellConstraints constraints) {
		return new JGoodiesConstraint(constraints);
	}

//---------------------------handling persistence
	private void initPersistenceDelegates() {
		PersistenceDelegate delegate = AbstractStylable.createPersistenceDelegate();
		// NOTE: its absolutely necessary to register per-class 
		// common ancestor will not do!
		Persist.addDefaultPersistenceDelegate(JGoodiesBorder.class,
				delegate);
		Persist.addDefaultPersistenceDelegate(JGoodiesRow.class,
				delegate);
		Persist.addDefaultPersistenceDelegate(JGoodiesColumn.class,
				delegate);
		Persist.addDefaultPersistenceDelegate(JGoodiesStyleMap.class,
		    JGoodiesStyleMap.createPersistenceDelegate());
	}


//---------------------------factory methods
	protected List createList() {
		return new ArrayList();
	}

//---------------------------specifics of jgoodies - direct access
	private RowSpec getRowSpec(String encodedString) {
		return new RowSpec(encodedString);
	}

	private ColumnSpec getColumnSpec(String encodedString) {
		return new ColumnSpec(encodedString);
	}

}
