package de.kleopatra.forms.description.jgoodies;

import java.awt.Component;
import java.awt.LayoutManager;
import java.util.*;

import javax.swing.border.Border;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.*;

import de.kleopatra.forms.description.*;

/** JGoodies Forms specific implementation of LayoutTranslator.
 *
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public final class JGoodiesLayoutTranslator implements LayoutTranslator {

//---------------------------translate generic elements into specific

	/** returns a constraint object that's understood by the created layoutManager.
	 */
	public Object createConstraintSpec(Constraint constraint) {
		if (constraint instanceof JGoodiesConstraint) {
			return ((JGoodiesConstraint) constraint).getCellConstraints();
		}
		return null;
	}

	public LayoutManager createLayoutManager(List rows, List columns) {
		FormLayout formLayout = createFormLayout(getColumnSpecs(columns),
															getRowSpecs(rows));
		int[][] rowGroups = createGroups(rows);
		if (rowGroups != null) {
			formLayout.setRowGroups(rowGroups);
		}
		int[][] columnGroups = createGroups(columns);
		if (columnGroups != null) {
			formLayout.setColumnGroups(columnGroups);
		}
		return formLayout;
	}

	public LayoutDebugPainter createLayoutDebugPainter() {
		return new FormLayoutDebugPainter();
	}

	public Border createBorder(LayoutBorder border) {
		if ((border == null) || (!border.isEnabled())) {
			return null;
		}
		return Borders.createEmptyBorder(border.getValidEncoding());
	}

//---------------------------translate specific elements into generic
	public boolean isCompatible(LayoutManager layoutManager) {
		return layoutManager instanceof FormLayout;
	}

	public Constraint createConstraint(Object constraintSpec) {
		if (constraintSpec instanceof CellConstraints) {
			return createJGoodiesConstraint((CellConstraints) constraintSpec);
		}
		return null;
	}

	// PENDING: not yet updated to LogicalLineType
	// used from reader only...
	public Row createRow(Object rowSpec) {
		if (rowSpec instanceof RowSpec) {
			return createJGoodiesRow((RowSpec) rowSpec);
		}
		return null;
	}

	public Column createColumn(Object columnSpec) {
		if (columnSpec instanceof ColumnSpec) {
			return createJGoodiesColumn((ColumnSpec) columnSpec);
		}
		return null;
	}

	public Constraint createConstraint(LayoutManager layoutManager,
			Component comp) {
		if (!(layoutManager instanceof FormLayout)) {
			return null;
		}
		return createConstraint(((FormLayout) layoutManager)
			.getConstraints(comp));
	}

	public List createRows(LayoutManager layoutManager) {
		if (!(layoutManager instanceof FormLayout)) {
			return null;
		}
		return createRowList((FormLayout) layoutManager);
	}

	public List createColumns(LayoutManager layoutManager) {
		if (!(layoutManager instanceof FormLayout)) {
			return null;
		}
		return createColumnList((FormLayout) layoutManager);
	}

//--------------------------- private helper: from generic to special
	private RowSpec[] getRowSpecs(List rows) {
		List result = createList();
		for (Iterator iterator = rows.iterator(); iterator.hasNext(); ) {
			Row row = (Row) iterator.next();
			if (row instanceof JGoodiesRow) {
				result.add(((JGoodiesRow) row).getRowSpec());
			}
		}
		return (RowSpec[]) result.toArray(new RowSpec[result.size()]);
	}

	private ColumnSpec[] getColumnSpecs(List columns) {
		List result = createList();
		for (Iterator iterator = columns.iterator(); iterator.hasNext(); ) {
			Column column = (Column) iterator.next();
			if (column instanceof JGoodiesColumn) {
				result.add(((JGoodiesColumn) column).getColumnSpec());
			}
		}
		return (ColumnSpec[]) result.toArray(new ColumnSpec[result.size()]);
	}

	private int[][] createGroups(List lines) {
		Map groups = new HashMap();
		for (int i = 0; i < lines.size(); i++) {
			Object groupID = ((Line) lines.get(i)).getGroupID();
			if (groupID != null) {
				List group = (List) groups.get(groupID);
				if (group == null) {
					group = createList();
					groups.put(groupID, group);
				}
				group.add(new Integer(i + 1));
			}
		}
		List allGroupArrays = createList();
		for (Iterator iterator = groups.values().iterator();
				iterator.hasNext(); ) {
			List group = (List) iterator.next();
			int[] array = new int[group.size()];
			for (int i = 0; i < array.length; i++) {
				array[i] = ((Integer) group.get(i)).intValue();
			}
			allGroupArrays.add(array);
		}
		int[][] result = new int[allGroupArrays.size()][];
		for (int i = 0; i < result.length; i++) {
			result[i] = (int[]) allGroupArrays.get(i);
		}
		return result;
	}

//--------------------------- private helper: from special to generic
	private List createRowList(FormLayout layout) {
		List result = createList();
		for (int i = 1; i <= layout.getRowCount(); i++) {
			result.add(createRow(layout.getRowSpec(i)));
		}
		addGroups(result, layout.getRowGroups());
		return result;
	}

	private List createColumnList(FormLayout layout) {
		List result = createList();
		for (int i = 1; i <= layout.getColumnCount(); i++) {
			result.add(createColumn(layout.getColumnSpec(i)));
		}
		addGroups(result, layout.getColumnGroups());
		return result;
	}

	private void addGroups(List lines, int[][] groups) {
		if (groups == null) {
			return;
		}
		for (int group = 0; group < groups.length; group++) {
			String groupID = "group" + group;
			int[] groupIndices = groups[group];
			for (int i = 0; i < groupIndices.length; i++) {
				Line line = (Line) lines.get(groupIndices[i] - 1);
				line.setGroupID(groupID);
			}
		}
	}

//---------------------------factory methods
	private FormLayout createFormLayout(ColumnSpec[] columnSpecs,
			RowSpec[] rowSpecs) {
		return new FormLayout(columnSpecs, rowSpecs);
	}

	protected Row createJGoodiesRow(RowSpec spec) {
		JGoodiesRow row = new JGoodiesRow();
		row.setEncodedString(FormUtils.getEncodedString(spec));
		return row;
	}

	protected Column createJGoodiesColumn(ColumnSpec spec) {
		JGoodiesColumn column = new JGoodiesColumn();
		column.setEncodedString(FormUtils.getEncodedString(spec));
		return column;
	}

	protected Constraint createJGoodiesConstraint(CellConstraints constraints) {
		return new JGoodiesConstraint(constraints);
	}

//---------------------------general factory methods
	protected List createList() {
		return new ArrayList();
	}
}
