package de.kleopatra.forms.description.jgoodies;

import java.awt.Component;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.*;

import de.kleopatra.forms.description.Line;
import de.kleopatra.forms.description.Style;

/** Wrapper of FormSpec as Line.
 *
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public abstract class JGoodiesLine extends AbstractStylable implements Line {

	protected final String DEL = ":";
	protected Object groupID;
	private PropertyChangeSupport propertySupport;

//---------------------- implementing Stylable
	public List getStyles() {
		List list = new ArrayList();
		list.addAll(getContentTypes());
		list.addAll(getGapTypes());
		list.add(getGlueType());
		return Collections.unmodifiableList(list);
	}

//---------------------------- implementing Line
	public Object getGroupID() {
		return groupID;
	}

	public void setGroupID(Object groupID) {
		Object oldID = getGroupID();
		this.groupID = groupID;
		firePropertyChange("groupID", oldID, getGroupID());
	}

	public boolean isConstant() {
		return getLineType().isConstant();
	}

	public int getPixelSize(Component comp) {
		return getLineType().getPixelSize(comp);
	}

	public boolean isGlue() {
		return getLineType().isGlue();
	}

	public boolean isGap() {
		return getLineType().isGap();
	}

	public String getMediumDescription() {
		String type = "Content: ";
		if (isGap()) {
			type = "Gap: ";
		}
		if (isGlue()) {
			type = "Glue: ";
		}
		return type + ((LineStyle) getType()).getMediumDescription()
					 + getLongGroupString();
	}

	public void addPropertyChangeListener(PropertyChangeListener l) {
		if (propertySupport == null) {
			propertySupport = new PropertyChangeSupport(this);
		}
		propertySupport.addPropertyChangeListener(l);
	}

	public void removePropertyChangeListener(PropertyChangeListener l) {
		if (propertySupport == null) {
			return;
		}
		propertySupport.removePropertyChangeListener(l);
	}

//------------------------ overriding super
	public String toString() {
		return getValidEncoding() + getGroupString();
	}

//------------------------helper
	protected void firePropertyChange(String name, Object oldValue,
			Object newValue) {
		if (propertySupport == null) {
			return;
		}
		propertySupport.firePropertyChange(name, oldValue, newValue);
	}

//------------------------------ abstract helpers
	protected abstract List getContentTypes();

	protected abstract List getGapTypes();

	protected abstract Object getGlueType();

//--------------------------- helpers
	protected void setStyle(Style style) {
		super.setStyle(style);
		firePropertyChange("encodedString", null, getValidEncoding());
	}

	protected LineStyle getLineType() {
		return (LineStyle) getStyle();
	}

	protected String getGroupString() {
		if ((getGroupID() != null)
				&& (String.valueOf(getGroupID()).length() > 0)) {
			return ", " + getGroupID();
		}
		return "";
	}

	protected String getLongGroupString() {
		if ((getGroupID() != null)
				&& (String.valueOf(getGroupID()).length() > 0)) {
			return ", Group: " + getGroupID();
		}
		return "";
	}
}
