/*
 * Created on 10.05.2004
 *
 */
package de.kleopatra.forms.description.jgoodies;

import java.beans.*;
import java.util.*;

import javax.swing.border.Border;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.RowSpec;

import de.kleopatra.forms.description.*;

/**
 *  JGoodies Forms specific implementation of LogicalTypeMap.
 * 
 *  Some notes:
 *  <ul>
 *  <li> isConfigurableXX() will return true for all typeKeys that
 *    are not contained in XX.PREDEFINED. 
 *  <li> all add/remove/rename methods will do nothing if isConfigurable()
 *   of the typeKey returns false.  
 *  <li> addXX(...) will create a new type. Depending on whether the typeKey
 *   is already known or not it will either replace the old entry with the
 *   new one or insert the new at the end of the list.
 *  
 *  </ul>
 *
 * @author (C) 2004, Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/07/01 12:39:59 $
 */
public class JGoodiesStyleMap implements StyleMap {

	private ColumnStyle columnGlueType;
	private RowStyle rowGlueType;
	private List rowContentTypeList;
	private List rowGapTypeList;
	private List columnContentTypeList;
	private List columnGapTypeList;
	private List borderTypeList;
	public JGoodiesStyleMap() {
		initRowTypes();
		initColumnTypes();
		initBorderTypes();
	}

//--------------------- accessing available logical types
	public List getRowContentStyles() {
		return Collections.unmodifiableList(getRowContents());
	}

	public List getRowGapStyles() {
		return Collections.unmodifiableList(getRowGaps());
	}

	public Style getRowGlueStyle() {
		if (rowGlueType == null) {
			rowGlueType = createRowType(Row.GLUE, FormFactory.GLUE_ROWSPEC, "glue");
		}
		return rowGlueType;
	}

	public List getColumnContentStyles() {
		return Collections.unmodifiableList(getColumnContents());
	}

	public List getColumnGapStyles() {
		return Collections.unmodifiableList(getColumnGaps());
	}

	public Style getColumnGlueStyle() {
		if (columnGlueType == null) {
			columnGlueType = createColumnType(Column.GLUE, FormFactory.GLUE_COLSPEC,
					"glue");
		}
		return columnGlueType;
	}

	public List getBorderStyles() {
		return Collections.unmodifiableList(getBorders());
	}

//--------------------------- adding logical types
	public void addRowContentStyle(Object typeKey, String encodedString,
			String description) {
		add(getRowContents(),
				createRowType(typeKey, encodedString, description));
	}

	public void addRowGapStyle(Object typeKey, String encodedString,
			String description) {
		add(getRowGaps(),
				createRowType(typeKey, encodedString, description));
	}

  public void addColumnContentStyle(Object typeKey, String encodedString,
			String description) {
		add(getColumnContents(),
				createColumnType(typeKey,encodedString,	description));
	}

	public void addColumnGapStyle(Object typeKey, String encodedString,
			String description) {
		add(getColumnGaps(),
				createColumnType(typeKey, encodedString,
					description));
	}


  public void addBorderStyle(Object typeKey, String encodedString,
			String description) {
		add(getBorders(), createBorderType(typeKey, encodedString, description));
	}

//----------------- removing logical types  
	public void removeRowContentStyle(Object typeKey) {
		if (!isPredefinedType(Row.PREDEFINED_TYPES, typeKey)) {
			removeByTypeKey(getRowContents(), typeKey);
		}
	}

	public void removeRowGapStyle(Object typeKey) {
		if (!isPredefinedType(Row.PREDEFINED_TYPES, typeKey)) {
			removeByTypeKey(getRowGaps(), typeKey);
		}
	}

	public void removeColumnContentStyle(Object typeKey) {
		if (!isPredefinedType(Column.PREDEFINED_TYPES, typeKey)) {
			removeByTypeKey(getColumnContents(), typeKey);
		}
	}

	public void removeColumnGapStyle(Object typeKey) {
		if (!isPredefinedType(Column.PREDEFINED_TYPES, typeKey)) {
			removeByTypeKey(getColumnGaps(), typeKey);
		}
	}

	public void removeBorderStyle(Object typeKey) {
		if (!isPredefinedType(LayoutBorder.PREDEFINED_TYPES, typeKey)) {
			removeByTypeKey(getBorders(), typeKey);
		}
	}


  public void renameRowContentStyle(Object oldTypeKey, Object newTypeKey) {
    renameRowStyle(getRowContents(), oldTypeKey, newTypeKey);
  }

  public void renameRowGapStyle(Object oldTypeKey, Object newTypeKey) {
    renameRowStyle(getRowGaps(), oldTypeKey, newTypeKey);
  }
	
	public void renameColumnContentStyle(Object oldTypeKey, Object newTypeKey) {
    renameColumnStyle(getColumnContents(), oldTypeKey, newTypeKey);
	}
  public void renameColumnGapStyle(Object oldTypeKey, Object newTypeKey) {
    renameColumnStyle(getColumnGaps(), oldTypeKey, newTypeKey);

  }

	
	public void renameBorderStyle(Object oldTypeKey, Object newTypeKey) {
    int i = findByTypeKey(getBorders(), oldTypeKey);
    if (i < 0) return;
    BorderStyle style = (BorderStyle) getBorders().get(i);
    getBorders().set(i, createBorderType(newTypeKey, style.getBorder(), style.getDescription()));
    
  }

  public Style getColumnContentStyle(Object typeKey) {

    
    return getStyle(getColumnContents(), typeKey);
  }


  public Style getColumnGapStyle(Object typeKey) {
    return getStyle(getColumnGaps(), typeKey);
  }

  public Style getRowContentStyle(Object typeKey) {
    return getStyle(getRowContents(), typeKey);
  }

  public Style getRowGapStyle(Object typeKey) {
    return getStyle(getRowGaps(), typeKey);
  }

  public Style getBorderStyle(Object typeKey) {
    return getStyle(getBorders(), typeKey);
  }

	
//-----------------------------
	
  public boolean isConfigurableRowStyle(Object typeKey) {
    return !isPredefinedType(Row.PREDEFINED_TYPES, typeKey);
  }

  public boolean isConfigurableColumnStyle(Object typeKey) {
    return !isPredefinedType(Column.PREDEFINED_TYPES, typeKey);
  }

  public boolean isConfigurableBorderStyle(Object typeKey) {
    return !isPredefinedType(LayoutBorder.PREDEFINED_TYPES, typeKey);
  }

//----------------------------- persistence
  
	public static PersistenceDelegate createPersistenceDelegate() {
		DefaultPersistenceDelegate delegate = new DefaultPersistenceDelegate() {

			protected void initialize(Class clazz, Object oldInstance,
					Object newInstance, Encoder out) {
				super.initialize(clazz, oldInstance, newInstance, out);
				JGoodiesStyleMap styleMap = (JGoodiesStyleMap) oldInstance;
				List columnContentTypes = styleMap.getColumnContentStyles();
				writeConfigurableColumns(oldInstance, out, columnContentTypes, "addColumnContentStyle");
				writeConfigurableColumns(oldInstance, out, styleMap.getColumnGapStyles(), "addColumnGapStyle");
				writeConfigurableRows(oldInstance, out, styleMap.getRowContentStyles(), "addRowContentStyle");
				writeConfigurableRows(oldInstance, out, styleMap.getRowGapStyles(), "addRowGapStyle");
				writeConfigurableBorders(oldInstance, out, styleMap.getBorderStyles(), "addBorderStyle");
			}

      private void writeConfigurableColumns(Object oldInstance, Encoder out, List styles, String method) {
				JGoodiesStyleMap styleMap = (JGoodiesStyleMap) oldInstance;
       for (Iterator iterator = styles.iterator(); iterator.hasNext();) {
          Style style = (Style) iterator.next();
          if (styleMap.isConfigurableColumnStyle(style.getLogicalType())) {
            out.writeStatement(new Statement(oldInstance, method,
                new Object[] { style.getLogicalType(), style.getEncodedString(), style.getDescription()}));
          }
          
        }
      }
      private void writeConfigurableRows(Object oldInstance, Encoder out, List styles, String method) {
				JGoodiesStyleMap styleMap = (JGoodiesStyleMap) oldInstance;
       for (Iterator iterator = styles.iterator(); iterator.hasNext();) {
          Style style = (Style) iterator.next();
          if (styleMap.isConfigurableRowStyle(style.getLogicalType())) {
            out.writeStatement(new Statement(oldInstance, method,
                new Object[] { style.getLogicalType(), style.getEncodedString(), style.getDescription()}));
          }
          
        }
      }
      private void writeConfigurableBorders(Object oldInstance, Encoder out, List styles, String method) {
				JGoodiesStyleMap styleMap = (JGoodiesStyleMap) oldInstance;
       for (Iterator iterator = styles.iterator(); iterator.hasNext();) {
          Style style = (Style) iterator.next();
          if (styleMap.isConfigurableBorderStyle(style.getLogicalType())) {
            out.writeStatement(new Statement(oldInstance, method,
                new Object[] { style.getLogicalType(), style.getEncodedString(), style.getDescription()}));
          }
          
        }
      }
		};
		return delegate;
	}

	//-------------------------------- helper code
	private boolean isPredefinedType(Object[] types, Object styles) {
	  for (int i = 0; i < types.length; i++) {
      if (styles.equals(types[i])) return true;
    }
		return false;
	}

  private Style getStyle(List styles, Object typeKey) {
    int i = findByTypeKey(styles, typeKey);
    return i < 0 ? null : (Style) styles.get(i);
  }
	
	private void removeByTypeKey(List list, Object typeKey) {
		for (Iterator iter = list.iterator(); iter.hasNext(); ) {
			Style element = (Style) iter.next();
			if (typeKey.equals(element.getLogicalType())) {
				iter.remove();
				return;
			}
		}
	}

	private void add(List list, Style type) {
		int inList = findByType(list, type);
		if (inList < 0) {
			list.add(type);
		} else {
			list.set(inList, type);
		}
	}

	private int findByType(List list, Style type) {
	  return findByTypeKey(list, type.getLogicalType());
	}

	private int findByTypeKey(List list, Object locigalType) {
	  for (int i = 0; i < list.size(); i++) {
			Style inList = (Style) list.get(i);
			if (inList.getLogicalType().equals(locigalType)) {
				return i;
			}
      
    }
	  return -1;
	}
  private void renameColumnStyle(List columnStyles, Object oldTypeKey, Object newTypeKey) {
    int i = findByTypeKey(columnStyles, oldTypeKey);
	  if (i < 0) return;
	  ColumnStyle style = (ColumnStyle) columnStyles.get(i);
	  columnStyles.set(i, createColumnType(newTypeKey, style.getColumnSpec(), style.getDescription()));
  }

  private void renameRowStyle(List rowStyles, Object oldTypeKey, Object newTypeKey) {
    int i = findByTypeKey(rowStyles, oldTypeKey);
	  if (i < 0) return;
	  RowStyle style = (RowStyle) rowStyles.get(i);
	  rowStyles.set(i, createRowType(newTypeKey, style.getRowSpec(), style.getDescription()));
  }

//------------------------------------- init
	protected void initRowTypes() {

		// init content 
		add(getRowContents(),
				createRowType(Row.DEFAULT_CONTENT, FormFactory.DEFAULT_ROWSPEC,
					"default"));
		add(getRowContents(),
				createRowType(Row.PREF_CONTENT, FormFactory.PREF_ROWSPEC,
					"preferred"));
		add(getRowContents(),
				createRowType(Row.MIN_CONTENT, FormFactory.MIN_ROWSPEC, "minimum"));

		// init gap
		add(getRowGaps(),
				createRowType(Row.RELATED_GAP, FormFactory.RELATED_GAP_ROWSPEC,
					"related"));
		add(getRowGaps(),
				createRowType(Row.UNRELATED_GAP, FormFactory.UNRELATED_GAP_ROWSPEC,
					"unrelated"));
		add(getRowGaps(),
				createRowType(Row.LINE_GAP, FormFactory.LINE_GAP_ROWSPEC, "line"));
		add(getRowGaps(),
				createRowType(Row.NARROW_LINE_GAP, FormFactory.NARROW_LINE_GAP_ROWSPEC,
					"narrow line"));
		add(getRowGaps(),
				createRowType(Row.PARAGRAPH_GAP, FormFactory.PARAGRAPH_GAP_ROWSPEC,
					"paragraph"));
	}

	protected void initColumnTypes() {

		// contents
		add(getColumnContents(),
				createColumnType(Column.DEFAULT_CONTENT, FormFactory.DEFAULT_COLSPEC,
					"default"));
		add(getColumnContents(),
				createColumnType(Column.PREF_CONTENT, FormFactory.PREF_COLSPEC,
					"preferred"));
		add(getColumnContents(),
				createColumnType(Column.MIN_CONTENT, FormFactory.MIN_COLSPEC,
					"minimum"));
		add(getColumnContents(),
				createColumnType(Column.BUTTON, FormFactory.BUTTON_COLSPEC,
					"button"));
		add(getColumnContents(),
				createColumnType(Column.GROWING_BUTTON,
					FormFactory.GROWING_BUTTON_COLSPEC, "growing button"));

		// gaps
		add(getColumnGaps(),
				createColumnType(Column.RELATED_GAP, FormFactory.RELATED_GAP_COLSPEC,
					"related"));
		add(getColumnGaps(),
				createColumnType(Column.UNRELATED_GAP, FormFactory.UNRELATED_GAP_COLSPEC,
					"unrelated"));
		add(getColumnGaps(),
				createColumnType(Column.LABEL_COMPONENT_GAP,
					FormFactory.LABEL_COMPONENT_GAP_COLSPEC, "label-component"));
	}

	protected void initBorderTypes() {
		add(getBorders(),
				createBorderType(LayoutBorder.NO_BORDER, (Border) null, "no border"));
		add(getBorders(),
				createBorderType(LayoutBorder.DIALOG, Borders.DIALOG_BORDER,
					"dialog"));
		add(getBorders(),
				createBorderType(LayoutBorder.TABBED_DIALOG, Borders.TABBED_DIALOG_BORDER,
					"tab-dialog"));
		add(getBorders(),
				createBorderType(LayoutBorder.BUTTON_BAR_GAP, Borders.BUTTON_BAR_GAP_BORDER,
					"buttonbar"));
		add(getBorders(),
				createBorderType(LayoutBorder.DLU2, Borders.DLU2_BORDER, "narrow"));
		add(getBorders(),
				createBorderType(LayoutBorder.DLU14, Borders.DLU14_BORDER, "broad"));
	}

//---------------------------- factory methods (plus lazy instantiation)
	
	protected RowStyle createRowType(Object type, RowSpec spec,
			String description) {
		return new RowStyle(type, spec, description);
	}

	protected ColumnStyle createColumnType(Object logicalType, ColumnSpec spec,
			String description) {
		return new ColumnStyle(logicalType, spec, description);
	}

	protected BorderStyle createBorderType(Object typeKey, Border border,
			String description) {
		return new BorderStyle(typeKey, (Borders.EmptyBorder) border, description);
	}

	private Style createRowType(Object typeKey, String encodedString, String description) {
    return createRowType(typeKey, new RowSpec(encodedString), description);
  }

	private Style createColumnType(Object typeKey, String encodedString, String description) {
    return createColumnType(typeKey, new ColumnSpec(encodedString),
				description);
  }
	
	protected BorderStyle createBorderType(Object logicalType, String encoding, String description) {
	  return createBorderType(logicalType, (Borders.EmptyBorder) Borders
				.createEmptyBorder(encoding), description);
	}
	
	protected List getRowContents() {
		if (rowContentTypeList == null) {
			rowContentTypeList = createList();
		}
		return rowContentTypeList;
	}

	protected List getRowGaps() {
		if (rowGapTypeList == null) {
			rowGapTypeList = createList();
		}
		return rowGapTypeList;
	}

	protected List getColumnContents() {
		if (columnContentTypeList == null) {
			columnContentTypeList = createList();
		}
		return columnContentTypeList;
	}

	protected List getColumnGaps() {
		if (columnGapTypeList == null) {
			columnGapTypeList = createList();
		}
		return columnGapTypeList;
	}

	protected List getBorders() {
		if (borderTypeList == null) {
			borderTypeList = createList();
		}
		return borderTypeList;
	}

	
	
	protected List createList() {
		return new ArrayList();
	}



}
