package de.kleopatra.forms.view;

import java.awt.Color;

import javax.swing.*;

import com.jgoodies.forms.layout.*;

import de.kleopatra.view.UIProperties;

/**     Horizontal JSeparator with title.
 *  <p>
 *    NOTE don't use resource but direct FormLayout to build!
 *    Otherwise it might not be able cope with custom storage formats in shared
 *    FormIO.
 *
 *
 *      @version $Revision: 1.1.1.1 $ $Date: 2004/08/19 11:18:34 $
 *      @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public class JTitledSeparator extends JPanel {

	protected JLabel titleLabel;
	protected JSeparator separator;
	public JTitledSeparator() {
		this(null);
	}

	public JTitledSeparator(String title) {
		initComponents();
		setText(title);
	}

	public void setText(String title) {
		if (!hasLabel() && (title == null)) {
			return;
		}
		getLabel().setText(title);
		updateLayout();
	}

	public String getText() {
		if (hasLabel()) {
			return getLabel().getText();
		}
		return null;
	}

	//---------------------------helper
	protected void updateLayout() {
		if ((getText() == null) || (getText().length() == 0)) {
			ensureGap(false);
		} else {
			ensureGap(true);
		}
	}

	private ColumnSpec gap = new ColumnSpec("l:4dlu");
	private void ensureGap(boolean useGap) {
		boolean hasGap = ((FormLayout) getLayout()).getColumnCount() == 3;
		if ((useGap &&!hasGap)) {
			((FormLayout) getLayout()).insertColumn(2, gap);
		} else if (!useGap && hasGap) {
			((FormLayout) getLayout()).removeColumn(2);
		}
	}

	//---------------------------init
	protected void initComponents() {
		separator = createSeparator("titleSpr");
		titleLabel = createLabel("titleLbl");

		// setting layout manually
		setLayout(new FormLayout("f:d:n, l:4dlu, f:d:g",
				"t:4dlu:n, c:d:n, t:4dlu:n"));
		CellConstraints cc = new CellConstraints();
		add(titleLabel, cc.xy(1, 2));
		add(separator, cc.xy(3, 2));

		// setting layout via resource

/*    add(titleLabel);
		add(separator);
		setName("titledSeparator");
		getFormWriter().rebuildForm(this);
*/
		setFocusable(false);
	}

	//---------------------------factory methods
/*
	protected FormWriter getFormWriter() {

		// will work as expected only if the shared factory uses the default
		// encoding ... hmmm.... should guarantee?
		return FormFactory.getFormWriter();
	}
*/
	protected boolean hasLabel() {
		return titleLabel != null;
	}

	protected JLabel getLabel() {
		if (titleLabel == null) {
			titleLabel = createLabel(null);
		}
		return titleLabel;
	}

	//---------------------------factory methods
	protected JLabel createLabel(String id) {
		JLabel label = new JTitleLabel();
		label.setName(id);
		return label;
	}

	protected JSeparator createSeparator(String id) {
		JSeparator result = new JSeparator();
		result.setName(id);
		return result;
	}

	// A label that uses the TitleBorder font and color
	// copied from jgoodies\forms DefaultComponentFactory
	private static class JTitleLabel extends JLabel {

		private JTitleLabel() {}

		private JTitleLabel(String text) {
			super(text);
			putClientProperty(UIProperties.NO_LABELENDING, Boolean.TRUE);
		}

		public void updateUI() {
			super.updateUI();
			Color foreground = UIManager.getColor("TitledBorder.titleColor");
			if (foreground != null) {
				setForeground(foreground);
			}
			setFont(UIManager.getFont("TitledBorder.font"));
		}
	}
	;
}
