/*
 * Created on 18.08.2004
 *
 */
package de.kleopatra.forms.view.builder;

import java.util.List;

import javax.swing.Action;
import javax.swing.JComponent;

import de.kleopatra.view.presentation.ActionFactory;

/**
 * Convenience implementation of Form.<p>
 * 
 * 
 * The component is created lazily in getFormComponent().
 * On first call this method will internally call
 * <ul>
 * <li> buildForm(...)
 * <li> bind(..)
 * <li> configure(..)
 * </ul>
 * 
 * Subclasses minimally have to provide a resource != null and
 * implement bind(..) to wire any binding/listeners/logic. See
 * the examples for code with/without using JGoodies Binding.
 * 
 * 
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.5 $ - $Date: 2004/09/29 14:40:19 $
 */
public abstract class AbstractForm implements Form {

  protected JComponent form;
  protected String formResource;
  protected Class formClass;
  protected AbstractForm() {}

  public AbstractForm(String formResource, Class formClass) {
    setResource(formResource, formClass);
  }

//-------------------- implementing Form

  /** returns the JComponent associated with the resource.
   *  PRE: getResource() != null.
   */
  public JComponent getFormComponent() {
    if (getResource() == null) {
      throw new IllegalStateException(
        "resource must be set before accessing form");
    }
    if (form == null) {
      form = buildForm(getResource());
      bind(form);
      configure(form);
    }
    return form;
  }

  public String getResource() {
    return formResource;
  }

  /** do nothing */
  public Object getTarget() {
    return null;
  }

  /** do nothing */
  public void setTarget(Object data) {
  }

  /**
   * does nothing...
   */
  public void commit() {
  }

  /**
   * does nothing...
   */
  public void flush() {}

  public void registerHandler(List actionIDs, Object handler) {
    if (getButtonResource() == null) {
      return;
    }
    ActionFactory.getInstance().registerHandler(actionIDs, getButtonResource(), handler);
  }

  /**
   *
   */
  public void registerActionsAsHandlers(List actionIDs, List actions) {
    for (int i = 0; i < actions.size(); i++) {
     Action action = (Action) actions.get(i);
      registerActionAsHandler(actionIDs.get(i), action);
    }
  }

  /** convenience to register a single action.
   * 
   * @param actionID
   * @param action
   */
  public void registerActionAsHandler(Object actionID, Action action) {
    if (getButtonResource() == null) {
      return;
    }
    ActionFactory.getInstance().registerActionAsHandler(actionID,
        getButtonResource(), action);
    
  }
//----------------------- for subclasses to override  

  /** called after the form is loaded.
   * 
   * @param form
   */
  protected abstract void bind(JComponent form);

  /** called after the form is loaded and the widgets are bound.
   * 
   * @param form
   */
  protected void configure(JComponent form) {
    
  }

//------------------------ accessing resource information
  
  protected void setResource(String resource, Class targetClass) {
    this.formResource = resource;
    this.formClass = targetClass;
  }

  /** returns an appropriate class to search the resource.
   *  here: returns formClass if explicitly set or getClass()
   * @return 
   */
  protected Class getResourceClass() {
    return formClass != null ? formClass : getClass();
  }

  /** resource for registering actions.
   *  Here: same as getResource()
   */
  protected String getButtonResource() {
    return getResource();
  }

//------------------------ helper
  protected JComponent buildForm(String resource) {
    JComponent comp =
      (JComponent) FormFactory.getFormLoader().load(resource,
        getResourceClass());
    return comp;
  }
}
