package de.kleopatra.forms.view.builder;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import de.kleopatra.forms.description.DefaultLayoutDescription;
import de.kleopatra.forms.description.LayoutDescription;
import de.kleopatra.persistence.Persist;
import de.kleopatra.view.presentation.DefaultPresentation;
import de.kleopatra.view.presentation.Presentation;

/** Default implementation of FormDescription.
 *
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *  @version $Revision: 1.1.1.1 $ - $Date: 2004/06/25 09:00:59 $
 */
public class DefaultFormDescription implements FormDescription {

  protected Presentation presentation;
  protected LayoutDescription layoutDescription;
  protected Object formID;

  // trying to remove from persistent - does not work
  transient protected boolean isDirty;
  static {
    Persist.setTransient(DefaultFormDescription.class, "clean");
  }

  // trying to remove from persistent - does not work
  // transient protected boolean clean; 
  transient protected PropertyChangeSupport propertySupport;
  public DefaultFormDescription() {}

  public DefaultFormDescription(Object formID) {
    setFormID(formID);
  }

  public String toString() {
    return String.valueOf(getFormID());
  }

//---------------------------FormDescription
  public Presentation getPresentation() {
    if (presentation == null) {
      presentation = createPresentation();
    }
    presentation.setPresentationID(getFormID());
    return presentation;
  }

  public void setPresentation(Presentation presentation) {
    this.presentation = presentation;
    presentation.setPresentationID(getFormID());
    setClean(false);
  }

  public LayoutDescription getLayoutDescription() {
    if (layoutDescription == null) {
      layoutDescription = createLayoutDescription();
    }
    layoutDescription.setLayoutID(getFormID());
    return layoutDescription;
  }

  public void setLayoutDescription(LayoutDescription description) {
    this.layoutDescription = description;
    description.setLayoutID(getFormID());
    setClean(false);
  }

  public Object getFormID() {
    return formID;
  }

  public void setFormID(Object formID) {
    Object oldFormID = getFormID();
    this.formID = formID;
    updateDescriptions();
    firePropertyChange("formID", oldFormID, getFormID());
  }

  public void setClean(boolean clean) {
    boolean oldClean = isClean();
    isDirty = !clean;
    firePropertyChange("clean", oldClean, isClean());
  }

  public boolean isClean() {
    return !isDirty;
  }

//---------------------------FormDescription - propertyChangeSupport
  public void addPropertyChangeListener(String property,
      PropertyChangeListener l) {
    if (propertySupport == null) {
      propertySupport = createPropertyChangeSupport();
    }
    propertySupport.addPropertyChangeListener(property, l);
  }

  public void removePropertyChangeListener(String property,
      PropertyChangeListener l) {
    if (propertySupport == null) {
      return;
    }
    propertySupport.removePropertyChangeListener(property, l);
  }

  protected void firePropertyChange(String property, boolean oldValue,
      boolean newValue) {
    if (propertySupport == null) {
      return;
    }
    propertySupport.firePropertyChange(property, oldValue, newValue);
  }

  protected void firePropertyChange(String property, Object oldValue,
      Object newValue) {
    if (propertySupport == null) {
      return;
    }
    propertySupport.firePropertyChange(property, oldValue, newValue);
  }

//---------------------------helper
  protected void updateDescriptions() {
    getPresentation().setPresentationID(getFormID());
    getLayoutDescription().setLayoutID(getFormID());
  }

//---------------------------factory methods
  protected Presentation createPresentation() {
    return new DefaultPresentation();
  }

  protected LayoutDescription createLayoutDescription() {
    return new DefaultLayoutDescription();
  }

  protected PropertyChangeSupport createPropertyChangeSupport() {
    return new PropertyChangeSupport(this);
  }
}
