/*
 * Created on 24.10.2003
 *
 */
package de.kleopatra.forms.view.builder;

import java.awt.Container;

import de.kleopatra.forms.description.LayoutDescription;
import de.kleopatra.view.factory.UIFactory;

/**
 *  extended loading options: from file and from LayoutDescription.
 *
 *  <p>
 *  loading from file is similar to loading from resource, except
 *  that the resource is looked up at a absolute (?) file path.
 *  Dependent resources (for placeHolders) are assumed to be
 *  in the same directory as the parent resource.
 *  Secondary resources for the presentation (f.i. icons)
 *  are searched in ResourceManager.userResourceDir.
 *
 *  <p>
 *  loading from a LayoutDescription directly allows to
 *  specify the factory to use for component creation.
 *  Dependent resources (for PlaceHolders' FormDescription)
 *  are searched in ResourceManager.userResourceDir
 *  <p>
 *  Client code is responsible for setting the resourceDir
 *  (THINK: should be?)
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/06/25 09:01:00 $
 */
public interface ExtFormLoader extends FormLoader {

  /** load, create, fill and customize a container from
   *  a resource located at a file location.
   *  Semantics are equivalent to load(..)
   */
  public Container loadFromFile(String filename);

  /** create and fill a container from the LayoutDescription
   *  using the given factory.
   *  <p>
   *  Uses shared UIFactory if factory == null.
   *
   */
  public Container create(LayoutDescription description, UIFactory factory);

  /** create and fill a container from the LayoutDescription,
   *  using the given factory, autoreplacing
   *  all placeHolder children.
   *  <p>
   *  Uses shared UIFactory if factory == null.
   *  <p>
   *  optionally customize the auto-replaced children.
   *  NOTE: The not-replaced children are not customized.
   *
   *  This is a very special-purpose method
   *  needed when the Presentation of the children is not
   *  yet available as a resource (f.i. in a Preview during
   *  form design)
   *
   *
   *
   */
  public Container createWithAutoReplace(LayoutDescription description,
      UIFactory factory, boolean customizePlaceHolders);
}
