/*
 * Created on 18.08.2004
 *
 */
package de.kleopatra.forms.view.builder;

import java.util.List;

import javax.swing.JComponent;

/**
 * Convenience "Form" to quickly get hold of a container with
 * bound widgets. It's usable f.i. with FormUtils to add/show in a JFrame
 * or with JFormDialog to add/show in a JDialog.<p>
 * 
 * Main responsibilities
 * <ul>
 * <li> build a container from a resource and bind it's child
 * widgets to the properties of target data </li>
 * <li> access and modify the handled target data </li>
 * <li> provide hook to register external actions/handlers </li>
 * </ul>
 * 
 * NOTE: this is work-very-much-in-progress and probably will change.
 * 
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.3 $ - $Date: 2004/09/28 11:04:38 $
 */
public interface Form {

  /** return the built container. <p>
   *  
   * @return a component built from a layout resource. 
   */
  JComponent getFormComponent();

  /** return the resource to load from. <p>
   * 
   * PENDING: really needed to expose publicly? Used only in FormUtils as 
   * default title ...
   * 
   * @return
   */
  String getResource();
  
  /** set the target data.
   * 
   * @param target
   */
  void setTarget(Object target);

  /** return the target data.
   * 
   * @return
   */
  Object getTarget();

  /** a hook to register callback methods in an
   *  external handler to buttons loaded by this form. <p>
   * @param list  a list of methods names 
   * @param handler the object implementing the methods
   */
  void registerHandler(List list, Object handler);

  /** a hook to register actions to buttons loaded by this form. <p>
   *  
   * 	NOTE: it's the responsibility of the calling code to 
   *  keep the two parameter lists in synch. <p>
   * 
   * @param actions  a list of actions
   * @param actionIDs  a list of corresponding button ids 
   */
  void registerActionsAsHandlers(List actions, List actionIDs);

  void commit();

  void flush();


}
