/*
 * Created on 26.09.2003
 *
 */
package de.kleopatra.forms.view.builder;

import de.kleopatra.forms.description.LayoutDescription;
import de.kleopatra.forms.description.StyleMap;
import de.kleopatra.persistence.IOHandler;
import de.kleopatra.support.HandlerMap;
import de.kleopatra.view.presentation.Presentation;

/**
 *  HandlerMap to map handled to handler classes for form io.
 *  <p>
 *  currently custom implementations need to provide a IOHandler for each
 *  of the following classes:
 *  <ul>
 *  <li> LayoutDescription
 *  <li> Presentation
 *  <li> FormDescription
 *  </ul>
 *
 *  PENDING: need some mechanisme to publish and guarantee the necessary
 *  IOHandlers
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/06/29 12:43:00 $
 */
public class FormIOHandlerMap extends HandlerMap {

  public IOHandler getIOHandler(Class handledClass) {
    return (IOHandler) getHandler(handledClass);
  }

  public void addIOHandler(String handledClass, String handlerClass) {
    addHandler(handledClass, handlerClass);
  }

  public void addIOHandler(Class handledClass, Class handlerClass) {
    addHandler(handledClass, handlerClass);
  }

//------------------------------helper overwrite super  
  protected boolean checkType(Object object) {
    return object instanceof IOHandler;
  }

  protected void initHandlers() {
    handlers = createMap();
    addHandler(LayoutDescription.class, LayoutDescriptionIOHandler.class);
    addHandler(Presentation.class, PresentationIOHandler.class);
    addHandler(FormDescription.class, FormDescriptionIOHandler.class);
    addHandler(StyleMap.class, StyleMapIOHandler.class);
  }
}
