package de.kleopatra.forms.view.builder;

import java.awt.Frame;

import javax.swing.JFileChooser;

import de.kleopatra.forms.description.LayoutDescription;
import de.kleopatra.forms.description.StyleMap;
import de.kleopatra.persistence.IOHandler;
import de.kleopatra.view.presentation.Presentation;
import de.kleopatra.view.util.FileChooserHandler;

/** Convenience helper to ease FileChooser handling.
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/23 13:43:33 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public class FormIOHelper {

  private static final int READ = JFileChooser.OPEN_DIALOG;
  private static final int WRITE = JFileChooser.SAVE_DIALOG;
  protected FormIO formIO;
  protected Frame owner;
  protected FileChooserHandler customizer;
  public void setOwner(Frame frame) {
    this.owner = frame;
  }

  /** set a custom formIO to use.
   * if null the helper uses the shared FormIO as returned by
   * FormFactory.
   */
  public void setFormIO(FormIO formIO) {
    this.formIO = formIO;
  }

//----------------------- LayoutDescription
  public String chooseLayoutFilename(boolean read) {
    return getFilename(read ? READ : WRITE, LayoutDescription.class);
  }

  public LayoutDescription readLayoutFile() {
    String filename = chooseLayoutFilename(true);
    if (filename == null) {
      return null;
    }
    return getFormIO().readLayoutDescription(filename);
  }

  public boolean writeLayoutFile(LayoutDescription description) {
    String filename = chooseLayoutFilename(false);
    if (filename == null) {
      return false;
    }
    getFormIO().persistLayoutDescription(description, filename);
    return true;
  }

//----------------------------- PresentationDescription
  public String choosePresentationFilename(boolean read) {
    return getFilename(read ? READ : WRITE, Presentation.class);
  }

  public Presentation readPresentation() {
    String filename = choosePresentationFilename(true);
    if (filename == null) {
      return null;
    }
    return getFormIO().readPresentation(filename);
  }

  public boolean writePresentation(Presentation presentation) {
    String filename = choosePresentationFilename(false);
    if (filename == null) {
      return false;
    }
    getFormIO().persistPresentation(presentation, filename);
    return true;
  }

//-------------------------------- FormDescription
  public String chooseFormDescriptionFilename(boolean read) {
    return getFilename(read ? READ : WRITE, FormDescription.class);
  }

  public FormDescription readFormDescription() {
    String filename = chooseFormDescriptionFilename(true);
    if (filename == null) {
      return null;
    }
    return getFormIO().readFormDescription(filename);
  }

  public boolean writeFormDescription(FormDescription formDescription) {
    String filename = chooseFormDescriptionFilename(false);
    if (filename == null) {
      return false;
    }
    getFormIO().persistFormDescription(formDescription, filename);
    return true;
  }

//-------------------------------- StyleMap
  public String chooseStyleMapFilename(boolean read) {
    return getFilename(read ? READ : WRITE, StyleMap.class);
  }

  public StyleMap readStyleMap() {
    String filename = chooseStyleMapFilename(true);
    if (filename == null) {
      return null;
    }
    return getFormIO().readStyleMap(filename);
  }

  public boolean writeStyleMap(StyleMap styleMap) {
    String filename = chooseStyleMapFilename(false);
    if (filename == null) {
      return false;
    }
    getFormIO().persistStyleMap(styleMap, filename);
    return true;
  }

//---------------------------helper
  protected String getFilename(int option, String ext) {
    return getFileChooserHandler().getFilename(option, ext);
  }

  protected String getFilename(int option, Class handledClass) {
    return getFilename(option,
        getFormIO().getIOHandler(handledClass).getExtension());
  }

  protected void customizeFileChooserHandler() {
    registerExtension(LayoutDescription.class);
    registerExtension(Presentation.class);
    registerExtension(FormDescription.class);
  }

  private void registerExtension(Class handledClass) {
    IOHandler handler = getFormIO().getIOHandler(handledClass);
    customizer.registerExtension(handler.getExtension(),
        handler.getDescription());
  }

  //---------------------------factory methods - lazy
  protected FormIO getFormIO() {
    if (formIO == null) {
      return FormFactory.getFormIO();
    }
    return formIO;
  }

  protected FileChooserHandler getFileChooserHandler() {
    if (customizer == null) {
      customizer = createFileChooserHandler();
      customizeFileChooserHandler();
    }
    customizer.setOwner(owner);
    return customizer;
  }

//---------------------------factory methods
  protected FileChooserHandler createFileChooserHandler() {
    return new FileChooserHandler();
  }
}
