/*
 * Created on 23.10.2003
 *
 */
package de.kleopatra.forms.view.builder;

import java.awt.Component;
import java.awt.Container;
import java.util.Collection;
import java.util.List;

import de.kleopatra.forms.description.StyleMap;

/**
 *  Responsibility to load a form from the classpath.
 *
 *  <p> load means:
 *  <ul>
 *  <li> find the resource via ResourceManager
 *  <li> create and fill the container, auto-replacing any
 *    direct PlaceHolder children. Note: indirect
 *    PlaceHolders are (not yet) supported.
 *  <li> customize the container
 *  </ul>
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.3 $ - $Date: 2004/09/30 10:18:21 $
 */
public interface FormLoader {

  /** set a custom formIO to use.
   * <p>
   * if null the writer uses the shared FormIO as returned by
   * FormFactory.
   */
  public void setFormIO(FormIO formIO);

  /** returns the formIO used by this loader.
   * <p>
   * will return shared FormIO of FormFactory if none is
   * set.
   * @return
   */

  /** create a container and load it from classpath resource.
   *
   */
  public Container load(String resourceName, Class targetClass);

  /** load the Container from classpath resource.
   *
   */
  public Container load(Container container, String resourceName,
      Class targetClass);

  /** re-arrange the direct children of a container from a resource defined by
   *  the container's name from a resource location relative to targetclass
   *  without customizing the contents.<p>
   *
   *
   * @param container - the container to re-arrange.
   * @param targetClass
   */
  public void rebuild(Container container, Class targetClass);

  /** re-arrange the direct children of the given container and
   *  the direct children of any placeholders from resources defined
   *  by their names from a resource location relative to targetClass. <p>
   *
   *  Customization of re-arranged components is turned on/off according
   *  to boolean parameter.
   *
   */
  public void rebuildAll(Container container, Class targetClass,
      boolean customize);

  /** replace a child of container with newComponent using the layoutDescription
   *  of container - as given by the resource with the container's name.
   *  <p>
   *  The replaced component is the child with the same name as the name of
   *  newComponent.
   *
   */
  public void replaceComponent(Container container, Class targetClass,
      Component component);

  /** replace a collection of children in container with a collection of new component
   *  using the layoutDescription
   *  of container - as given by the resource with the container's name.
   *  <p>
   *  The replaced components are the children with the same names as the names of components
   *  in the list.
   *
   * @param container
   * @param targetClass to find a formDescription relative to a classpath if needed
   * @param replacingComponents collection of replacing components
   */
  public void replaceComponents(Container container, Class targetClass,
      Collection replacingComponents);

  /** experimental: use the resource as template, use as children those in the
   *  collection instead of those defined in the resource.
   *
   *  NOTE: constraints in the resource that have no corresponding child
   *  in the collection are ignored.
   *
   */
  public Container loadTemplate(String resourceName, Class targetClass,
      Collection children);

  /** customize a list of actions from a resource.
   *  <p>
   *  NOTE: this couples implicitely into Swing if used.
   *
   */
  public void customizeActions(List actions, String resourceName,
      Class targetClass);

  /** load a StyleMap from a classpath resource.
   *
   * @param resourceName
   * @param clazz
   * @return
   */
  public StyleMap loadStyleMap(String resourceName, Class clazz);
}
