package de.kleopatra.forms.view.builder;

import java.awt.*;
import java.util.*;
import java.util.List;

import de.kleopatra.forms.description.*;
import de.kleopatra.view.factory.ComponentTypeMap;
import de.kleopatra.view.factory.UIFactory;
import de.kleopatra.view.presentation.*;

/** FormReader responsible for building descriptions from Containers.
 *
 *
 *  @deprecated - Currently not used. Maybe not in synch.
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public class FormReader {

  // side-effect: re-names children if getName() == null
  public LayoutDescription readLayoutDescription(Container container) {

    LayoutTranslator factory = getLayoutTranslator();
    if (!factory.isCompatible(container.getLayout())) {
      return null;
    }
    LayoutManager formLayout = container.getLayout();
    Component[] children = container.getComponents();
    List columns = factory.createColumns(formLayout);
    List rows = factory.createRows(formLayout);
    if (container.getName() == null) {
      container.setName(createName(containerCount, container));
    }
    List constraints = createList();
    ComponentTypeMap types = getComponentTypeMap(container);
    for (int i = 0; i < children.length; i++) {
      Constraint constraint = factory.createConstraint(formLayout,
                                children[i]);
      Class componentClass = children[i].getClass();
      String componentType = componentClass.getName();
      if (types != null) {
        componentType =
          String.valueOf(types.getComponentType(componentClass));
      }
      constraint.setComponentClassName((String) componentType);
      if ((children[i].getName() == null)
          || (children[i].getName().length() == 0)) {
        children[i].setName(createName(i, children[i]));
      }
      constraint.setComponentID(children[i].getName());
      constraints.add(constraint);
    }
    DefaultLayoutDescription description = createDefaultLayoutDescription();
    description.setLayoutID(container.getName());
    description.setRows(rows);
    description.setColumns(columns);
    description.setListConstraints(constraints);
    return description;
  }

/*  public List readPresentationList(Container container) {
    List result = createList();
    Component[] children = container.getComponents();
    PresentationManager manager = getPresentationManager();
    for (int i = 0; i < children.length; i++) {
      result.add(manager.getPresentationDescription(children[i]));
    }
    return result;
  }
*/
  public Presentation readPresentation(Container container) {
    List descriptions = createList();
    Component[] children = container.getComponents();
    PresentationManager manager = getPresentationManager();
    for (int i = 0; i < children.length; i++) {
      descriptions.add(manager.createPresentationDescription(children[i]));
    }
    DefaultPresentation result = createDefaultPresentation();
    result.setPresentationID(container.getName());
    result.setPresentationDescriptions(descriptions);
    return result;
  }

  public FormDescription readFormDescription(Container container) {
    LayoutDescription layout = readLayoutDescription(container);
    Presentation presentation = readPresentation(container);
    FormDescription desc = createFormDescription(layout.getLayoutID());
    desc.setLayoutDescription(layout);
    desc.setPresentation(presentation);
    return desc;
  }

//---------------------------helper
  private int containerCount;
  private String createName(int index, Component comp) {
    Object name = guessNameFromFactory(comp);
    if (name != null) {
      return String.valueOf(name);
    }
    String clazzname = comp.getClass().getName();
    int lastPoint = clazzname.lastIndexOf(".");
    if (lastPoint > 0) {
      clazzname = clazzname.substring(lastPoint + 1, clazzname.length());
    }
    return clazzname + index;
  }

  private Object guessNameFromFactory(Component comp) {
    return getUIFactory().getComponentID(comp);
  }

//---------------------------factory methods
  protected LayoutTranslator getLayoutTranslator() {
    return LayoutBuilderFactory.getLayoutTranslator();
  }

  protected PresentationManager getPresentationManager() {
    return getUIFactory().getPresentationManager();
  }

  protected DefaultLayoutDescription createDefaultLayoutDescription() {
    return new DefaultLayoutDescription();
  }

  protected DefaultPresentation createDefaultPresentation() {
    return new DefaultPresentation();
  }

  protected ComponentTypeMap getComponentTypeMap(Container container) {
    return getUIFactory().getComponentTypeMap();
  }

  protected FormDescription createFormDescription(Object formID) {
    return new DefaultFormDescription(formID);
  }

  protected UIFactory getUIFactory() {
    return FormFactory.getUIFactory();
  }

  protected List createList() {
    return new ArrayList();
  }

  protected Map createMap() {
    return new HashMap();
  }
}
