package de.kleopatra.forms.view.builder;

import java.awt.Component;
import java.awt.Container;
import java.util.*;

import javax.swing.JComponent;
import javax.swing.border.Border;

import de.kleopatra.forms.description.*;
import de.kleopatra.view.factory.UIFactory;

/** Default implementation of UIBuilder.
 *
 *  @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
class FormUIBuilder implements UIBuilder {

  public void fill(Container container, LayoutDescription layoutDescription,
      UIFactory uiFactory) {
    LayoutTranslator translator = getLayoutTranslator();
    container.removeAll();
    container
      .setLayout(translator.createLayoutManager(layoutDescription.getRows(),
        layoutDescription.getColumns()));
    for (Iterator constraints = layoutDescription.getConstraints().iterator();
        constraints.hasNext(); ) {
      Constraint constraint = (Constraint) constraints.next();
      Component comp = uiFactory.createComponent(constraint.getComponentID(),
                         constraint.getComponentClassName());
      container.add(comp, translator.createConstraintSpec(constraint));
    }
    container.setName(String.valueOf(layoutDescription.getLayoutID()));
    setBorder(container, layoutDescription);
  }

  public void refill(Container container,
      LayoutDescription layoutDescription) {
    LayoutTranslator translator = getLayoutTranslator();
    Map children = getChildren(container);
    container.removeAll();
    container
      .setLayout(translator.createLayoutManager(layoutDescription.getRows(),
        layoutDescription.getColumns()));
    for (Iterator constraints = layoutDescription.getConstraints().iterator();
        constraints.hasNext(); ) {
      Constraint constraint = (Constraint) constraints.next();
      Component comp = (Component) children.get(constraint.getComponentID());
      if (comp != null) {
        container.add(comp, translator.createConstraintSpec(constraint));
      }
    }
    setBorder(container, layoutDescription);
  }

  public void replaceComponents(Container container,
      LayoutDescription layoutDescription, Collection replacingChildren) {
    LayoutTranslator translator = getLayoutTranslator();
    Map children = getChildren(container);
    if (replacingChildren != null) {
      replaceChildren(children, replacingChildren);
    }
    container.removeAll();
    container
      .setLayout(translator.createLayoutManager(layoutDescription.getRows(),
        layoutDescription.getColumns()));
    for (Iterator constraints = layoutDescription.getConstraints().iterator();
        constraints.hasNext(); ) {
      Constraint constraint = (Constraint) constraints.next();
      Component comp = (Component) children.get(constraint.getComponentID());
      if (comp != null) {
        container.add(comp, translator.createConstraintSpec(constraint));
      }
    }
    setBorder(container, layoutDescription);
  }

//---------------------------helper

  /** puts the items of replacingChildren into the map children
   *  PRE: children != null AND replacingChildren != null
   */
  private void replaceChildren(Map children, Collection replacingChildren) {
    for (Iterator iter = replacingChildren.iterator(); iter.hasNext(); ) {
      Component element = (Component) iter.next();
      if ((element.getName() != null) && (element.getName().length() > 0)) {
        children.put(element.getName(), element);
      }
    }
  }

  protected Map getChildren(Container container) {
    Map children = createMap();
    for (int i = 0; i < container.getComponentCount(); i++) {
      Component comp = container.getComponent(i);
      if (comp.getName() != null) {
        children.put(comp.getName(), comp);
      }
    }
    return children;
  }

  private void setBorder(Container container,
      LayoutDescription layoutDescription) {
    if (!(container instanceof JComponent)) {
      return;
    }
    Border border =
      getLayoutTranslator().createBorder(layoutDescription.getBorder());
    ((JComponent) container).setBorder(border);

    //  Border readBorder = ((JComponent) container).getBorder();
  }

//---------------------------factory methods
  protected LayoutTranslator getLayoutTranslator() {
    return LayoutBuilderFactory.getLayoutTranslator();
  }

  protected Map createMap() {
    return new HashMap();
  }
}
