/*
 * Created on 15.06.2004
 *
 */
package de.kleopatra.forms.view.builder;

import java.awt.Component;
import java.awt.Dimension;
import java.util.Arrays;
import java.util.List;

import javax.swing.*;

import de.kleopatra.view.presentation.ActionFactory;

/**
 * A bunch of static utility methods.
 *
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.5 $ - $Date: 2004/10/04 12:18:34 $
 */
public class FormUtils {

  /** create a JFrame around the Form. <p>
   * 
   * pack and show if defaultShow is true. 
   * As a side-effect OptionPane's rootFrame is set to the frame.
   */ 
  public static JFrame createFrame(Form form, boolean defaultShow) {
    return createFrame(form, form.getResource(), defaultShow);
  }

  /** create a JFrame around the Form. <p>
   * 
   * pack and show if defaultShow is true. 
   * As a side-effect OptionPane's rootFrame is set to the frame.
   */ 
  public static JFrame createFrame(Form form, String title, boolean defaultShow) {
    return createFrame(form.getFormComponent(), title, defaultShow);
  }

  /** create a JFrame with content as contentPane, given title. <p>
   * 
   * pack and show if defaultShow is true. 
   * As a side-effect OptionPane's rootFrame is set to the frame.
   * 
   * @param content
   * @param title
   * @param defaultShow
   * @return
   */
  public static JFrame createFrame(JComponent content, String title, boolean defaultShow) {
    JFrame frame = new JFrame(title);
    JOptionPane.setRootFrame(frame);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.getContentPane().add(content);
    if (defaultShow) {
      frame.pack();
      frame.setLocationRelativeTo(null);
      frame.show();
    }
    return frame;
    
  }
  
  /**
   * Locates the given component on the screen's center.
   *
   * @param component   the component to be centered
   */
  public static void locateOnScreenCenter(Component component) {
    Dimension paneSize = component.getSize();
    Dimension screenSize = component.getToolkit().getScreenSize();
    component.setLocation((screenSize.width - paneSize.width) / 2,
        (screenSize.height - paneSize.height) / 2);
  }

  /** 
   * register actions and callback methods of handler
   * with the ActionFactory,
   * and customize their presentation attributes 
   * from the resource.
   * 
   * @param actionIDs an array of actionIDs 
   * @param resourceID the name of the resource 
   * @param handler  an object providing the callback methods
   * @return a list of all actions registered for the given resourceID
   */
  public static List installActions(Object[] actionIDs, String resourceID,
      Object handler) {
    return installActions(Arrays.asList(actionIDs), resourceID, handler);
  }

  /** 
   * register actions and callback methods of handler
   * with the ActionFactory,
   * and customize their presentation attributes 
   * from the resource.
   * 
   * @param actionIDs a list of actionIDs 
   * @param resourceID the name of the resource 
   * @param handler  an object providing the callback methods
   * @return a list of all actions registered for the given resourceID
   */
  public static List installActions(List actionIDs, String resourceID,
      Object handler) {
    ActionFactory factory = ActionFactory.getInstance();
    List list = factory.registerHandler(actionIDs, resourceID, handler);
    FormFactory.getFormLoader().customizeActions(list, resourceID,
        handler.getClass());
    return list;
  }

}
