/*
 * Created on 23.10.2003
 *
 */
package de.kleopatra.forms.view.builder;

import java.awt.Container;
import java.io.File;
import java.util.*;

import de.kleopatra.forms.description.LayoutDescription;
import de.kleopatra.view.factory.UIFactory;

/**
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.3 $ - $Date: 2004/09/30 10:18:21 $
 */
class PreviewFormLoader extends DefaultFormLoader implements ExtFormLoader {

//---------------- additional methods to handle file access
  public Container loadFromFile(String filename) {
    FormDescription formDescription =
      getFormIO().readFormDescription(filename);
    Container form = null;
    if (formDescription != null) {
      form = load(formDescription, getUIFactory(), null, true);
    }
    if (form != null) {
      LayoutDescription layoutDescription =
        formDescription.getLayoutDescription();
      readAndLoadPlaceHolders(form, layoutDescription, filename);

//      findAndLoadButtonPanels(form, layoutDescription);
    }
    return form;
  }

  //-----------------------additional methods to handle LayoutDescription
  public Container create(LayoutDescription description, UIFactory factory) {
    if (factory == null) {
      factory = getUIFactory();
    }
    return super.create(description, factory);
  }

  public Container createWithAutoReplace(LayoutDescription description,
      UIFactory factory, boolean customizePlaceHolders) {
    Container container = create(description, factory);
    if (container != null) {
      List placeHolders = findPlaceHoldersNames(description);

      // PENDING: should use given factory instead of default
      // PENDING: where to look for relative resources in PresentationDescription?
      // need some way to find relative to description 
      // but: used in Preview during building descriptions only - so it might be 
      // okay to assume that the client has set the ResourceManager.userResourceDir
      // to some reasonable value... 
      List substituents = findAndLoadAll(placeHolders, factory, null,
                            customizePlaceHolders);
      getUIBuilder().replaceComponents(container, description, substituents);

//      findAndLoadButtonPanels(container, description);
    }
    return container;
  }

//-----------------------helper to actually read
  private Container readAndLoad(String filename, UIFactory factory,
      Class targetClass, boolean customize) {
    FormDescription formDescription =
      getFormIO().readFormDescription(filename);
    if (formDescription != null) {
      return load(formDescription, factory, targetClass, customize);
    }
    return null;
  }

  /** build a list of forms from a list of filenames.
   *
   * @param descriptionNames list of filenames
   * @param customize the forms are customized if true
   * @return list of containers
   */
  private List readAndLoadAll(List fileNames, UIFactory factory,
      Class targetClass, boolean customize) {
    List forms = createList();
    for (Iterator iterator = fileNames.iterator(); iterator.hasNext(); ) {
      String filename = (String) iterator.next();
      Container form = readAndLoad(filename, factory, targetClass, customize);
      if (form != null) {
        forms.add(form);
      }
    }
    return forms;
  }

  private void readAndLoadPlaceHolders(Container form,
      LayoutDescription layoutDescription, String filename) {
    List placeHolders = findPlaceHoldersNames(layoutDescription);
    expandPaths(placeHolders, filename);
    List substituents = readAndLoadAll(placeHolders, getUIFactory(), null,
                          true);
    replaceComponents(form, (Class) null, substituents);
  }

//  private void readAndLoadAllButtonPanels(Container form, LayoutDescription layoutDescription, String filename) {
//    // TODO Auto-generated method stub
//    
//  }
//--------------------- helpers

  /** expands the ids to the same filepath as filename.
   *
   * @param placeHolders list of resourceIDs
   * @param filename file with the path to expand to
   */
  private void expandPaths(List placeHolders, String filename) {
    File file = new File(filename);
    String name = file.getName();
    String parent = file.getParent();
    for (ListIterator iterator = placeHolders.listIterator();
        iterator.hasNext(); ) {
      String element = (String) iterator.next();
      File created = new File(parent, element);
      iterator.set(created.getPath());
    }
  }
}
