package de.kleopatra.forms.view.builder;

import java.awt.Container;
import java.util.Collection;

import de.kleopatra.forms.description.LayoutDescription;
import de.kleopatra.view.factory.UIFactory;

/** Builder to fill a container according to a layout description.
 *
 *
 * @version $Revision: 1.2 $ $Date: 2004/09/30 10:18:21 $
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public interface UIBuilder {

  /** fills a container with components according to the layoutDescription.
   *  Uses the uiFactory to create the children.
   *  Guarantees that the container will have the newly created children only.
   */
  public void fill(Container comp, LayoutDescription layout,
      UIFactory uiFactory);

  /** fills a container with its own comps according to the layoutDescription.
   *  <p>
   * To work correctly - all children of the container need to be named
   *  (interpreted as componentID) and match the id of the constraints in the description.
   */
  public void refill(Container comp, LayoutDescription layout);

  /** fills a container with its own comps according to the layoutDescription, replacing
   *   those with names that match one contained in newChildren with the new.
   * <p>
   * NOTE: as a side-effect only children/replacing components which have a corresponding
   * constraint (that is one with the same id as comps) are (re-)inserted.
   * <p>
   *  NOTE: this is experimental, mainly to ease client handling of placeholders as long
   *  as there is not yet any support for hierarchical forms.
   *
   * @param container - the container
   * @param layout - the layoutDescription to use
   * @param newChildren - collection of Children to replace
   */
  public void replaceComponents(Container container,
      LayoutDescription layout, Collection newChildren);
}
