/*
 * Created on 26.09.2003
 *
 */
package de.kleopatra.persistence;

import java.io.InputStream;

/** IOHandler that uses the default XMLEncoder/XMLDecoder.
 * 
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.2 $ - $Date: 2004/09/30 10:15:13 $
 */
public abstract class AbstractIOHandler implements IOHandler {

//------------------------IOHandler

	public void write(Object object, String filename) {
		if (!checkType(object)) return;
		Persist.save(ensureExtension(filename), object);
	}

	public Object read(String filename) {
		Object result = Persist.load(ensureExtension(filename));
		if (checkType(result)) return result;
		return null;
	}

	public Object readResource(String resourceName, Class targetClass) {
		InputStream in = ResourceManager.getInstance()
			.getResourceAsStream(ensureExtension(resourceName), targetClass);
		if (in != null) {
			Object result = Persist.load(in);
			if (checkType(result)) return result;
		}
		return null;
	}

	public String ensureExtension(String filename) {
		filename = filename.trim();
		int lastPoint = getExtensionPosition(filename);
		if (lastPoint > 0) {
			filename = filename.substring(0, lastPoint);
		}
		return filename + getExtension();
	}

//----------------------------helper to be implemented by subclasses

	/** checks and returns whether type of object is compatible with 
	 * 	getHandledType.
	 * 	
	 * 
	 */
	protected abstract boolean checkType(Object object);

//-----------------------------helper

	protected int getExtensionPosition(String filename) {
		int lastPoint = filename.lastIndexOf(".");
		if (lastPoint >= 0) {
		  int distanceToEnd = filename.length() - lastPoint;
		  if (distanceToEnd > getExtension().length()) {
		    lastPoint = -1;
		  }
		}
		return lastPoint;
	}

}	
