/*
 * Created on 26.09.2003
 *
 */
package de.kleopatra.persistence;

/**
 * Generic handler for storing/loading objects to/from files and loading
 * form a resource.
 * <p>
 * It's responsible for one class - clients can rely on safely doing a 
 * type-cast on a read Object like:
 * 
 * <pre>
 * 	Class myClass = getHandledClass();
 * 	MyObject myObject = (MyObject) myClass.newInstance(); 
 * </pre>
 * 
 * file/resourcenames are forced to the fileExtension on read/write. 
 * 
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2003/12/02 14:45:40 $
 */
public interface IOHandler {
	/** returns the class this handler feels responsible for.
	 * 
	 */
	Class getHandledClass();

	/** the fileExtension that should be used to read/write this type.
	 * 
	 */
	String getExtension();

	/** returns the given filename with getFileExtension.
	 * 
	 */
	String ensureExtension(String filename);
	/** a description for use in FileFilters.
	 * 
	 */
	String getDescription();

	/** tries to write the given object to a file with filename.
	 *  if object's type is incompatible or an error occurs nothing is done
	 *
	 *	PENDING: use exceptions in production version?
	 *
	 */
	void write(Object object, String filename);

	/** read on object of type getHandledClass() to file with filename
	 *  returns null if object's type is incompatible or an error occured.
	 */
	Object read(String filename);

	/** read on object of type getHandledClass() to file with filename
	 *  returns null if object's type is incompatible or an error occured.
	 * @param resourceName
	 * @param targetClass class to use for resolving relative resource location
	 * 		locations are resolved as defined by ResourceManager.
	 */
	Object readResource(String resourceName, Class targetClass);
	
}
