package de.kleopatra.persistence;

import java.beans.*;
import java.io.*;
import java.util.*;
import java.util.logging.Logger;

/** Convenience to help persistence via XMLEncoder/Decoder. <p>
 *  PENDING: exceptions are swallowed.
 *
 *
 *  @version $Revision: 1.1.1.1 $ $Date: 2004/08/19 11:18:34 $
 *
 *  @author (C) 2003 Jeanette Winzenburg, Berlin
 *
 */
public class Persist {

	private static final String CLONE_FILE = "__temp.xxx";
	private static Map persistenceDelegates;
	
	public static void setTransient(Class persisteeClass, String property) {
	  BeanInfo info = null;
    try {
      info = Introspector.getBeanInfo(persisteeClass);
    } catch (IntrospectionException ex) {
      Logger.global.warning("Not able to get BeanInfo when trying to set as transient " + property);
      ex.printStackTrace();
      return;
    }
    PropertyDescriptor[] descriptors = info.getPropertyDescriptors();
	  for (int i = 0; i < descriptors.length; i++) {
      PropertyDescriptor desc = descriptors[i];
      if (desc.getName().equals(property)) {
        desc.setValue("transient", Boolean.TRUE);
        return;
      }
    }
	}

	public static void addDefaultPersistenceDelegate(Class clazz,
			PersistenceDelegate delegate) {
		if (persistenceDelegates == null) {
			persistenceDelegates = new HashMap();
		}
		persistenceDelegates.put(clazz, delegate);
	}

	public static void save(String filename, Object object) {
		try {
			save(new FileOutputStream(filename), object);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void save(String filename, Object object, Map delegates) {
		try {
			save(new FileOutputStream(filename), object, delegates);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static Object load(String filename) {
		Object result = null;
		try {
			return load(new FileInputStream(filename));
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
	}

	public static void save(OutputStream out, Object object) {
		try {
			XMLEncoder e = getXMLEncoder(out, persistenceDelegates);
			e.writeObject(object);
			e.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void save(OutputStream out, Object object, Map delegates) {
		try {
			XMLEncoder e = getXMLEncoder(out, delegates);
			e.writeObject(object);
			e.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static Object load(InputStream in) {
		Object result = null;
		try {
			XMLDecoder d = new XMLDecoder(new BufferedInputStream(in));
//			d.setExceptionListener(getExceptionListener());
			result = d.readObject();
			d.close();
		} catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}
		return result;
	}

	public static Object clone(Object object) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		save(out, object);
		ByteArrayInputStream in = new ByteArrayInputStream(out.toByteArray());
		return load(in);
	}

	public static ExceptionListener getExceptionListener() {

/*    if (exceptionListener == null) {
			exceptionListener = createExceptionListener();
		}
		return exceptionListener;
*/
		return createExceptionListener();
	}

	public static ExceptionListener createExceptionListener() {
		ExceptionListener l = new ExceptionListener() {

			public void exceptionThrown(Exception e) {
				System.out.println("in de/encoder: " + e);
				e.printStackTrace();
			}
		};
		return l;
	}

	private static XMLEncoder getXMLEncoder(OutputStream out, Map delegates) {
		XMLEncoder encoder = new XMLEncoder(new BufferedOutputStream(out));
		encoder.setExceptionListener(getExceptionListener());
		if (delegates != null) {
			for (Iterator iterator = delegates.keySet().iterator();
					iterator.hasNext(); ) {
				Class clazz = (Class) iterator.next();
				encoder.setPersistenceDelegate(clazz,
						(PersistenceDelegate) delegates.get(clazz));
			}
		}
		return encoder;
	}
}
