/*
 * Created on 26.11.2003
 *
 */
package de.kleopatra.support;


import java.io.Serializable;

import javax.swing.event.*;

/** Support for handling listener notification for CellEditors.
 *
  * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2003/12/02 14:45:40 $
 *
 */

public class CellEditorSupport implements Serializable {

		protected EventListenerList listenerList = new EventListenerList();
		transient protected ChangeEvent changeEvent = null;
		protected Object source;


		public CellEditorSupport(Object editor) {
			this.source = editor;
		}

		/**
		 * Adds a <code>CellEditorListener</code> to the listener list.
		 * @param l  the new listener to be added
		 */
		public void addCellEditorListener(CellEditorListener l) {
				listenerList.add(CellEditorListener.class, l);
		}

		/**
		 * Removes a <code>CellEditorListener</code> from the listener list.
		 * @param l  the listener to be removed
		 */
		public void removeCellEditorListener(CellEditorListener l) {
				listenerList.remove(CellEditorListener.class, l);
		}

		/*
		 * Notifies all listeners that have registered interest for
		 * notification on this event type.  The event instance
		 * is lazily created using the parameters passed into
		 * the fire method.
		 * @see EventListenerList
		 */
		public void fireEditingStopped() {
				// Guaranteed to return a non-null array
				Object[] listeners = listenerList.getListenerList();
				// Process the listeners last to first, notifying
				// those that are interested in this event
				for (int i = listeners.length-2; i>=0; i-=2) {
						if (listeners[i]==CellEditorListener.class) {
								// Lazily create the event:
								if (changeEvent == null)
										changeEvent = new ChangeEvent(source);
								((CellEditorListener)listeners[i+1]).editingStopped(changeEvent);
						}
				}
		}

		/*
		 * Notifies all listeners that have registered interest for
		 * notification on this event type.  The event instance
		 * is lazily created using the parameters passed into
		 * the fire method.
		 * @see EventListenerList
		 */
		public void fireEditingCanceled() {
				// Guaranteed to return a non-null array
				Object[] listeners = listenerList.getListenerList();
				// Process the listeners last to first, notifying
				// those that are interested in this event
				for (int i = listeners.length-2; i>=0; i-=2) {
						if (listeners[i]==CellEditorListener.class) {
								// Lazily create the event:
								if (changeEvent == null)
										changeEvent = new ChangeEvent(source);
								((CellEditorListener)listeners[i+1]).editingCanceled(changeEvent);
						}
				}
		}
}