/*
 * Created on 05.09.2003
 *
 */
package de.kleopatra.support;

import java.util.HashMap;
import java.util.Map;

import de.kleopatra.persistence.ResourceManager;

/**
 * a generic map for storing handlers for objects of a given class.
 *
 *	@version $Revision: 1.2 $ $Date: 2004/09/28 11:04:38 $
 * @author (C) Jeanette Winzenburg, Berlin
 */
public class HandlerMap  {

	protected Map handlers;
	
	public HandlerMap() {
		initHandlers();
	}

//----------------------------assesing handlers
	/** returns a handler for the given class.
	 * 
	 */
	public Object getHandler(Class valueClass) {
		Class usableValueClass = getUsableValueClass(valueClass);
		return handlers.get(usableValueClass);
	}

	/** stores handler for a given handledClass. 
	 * an existing handler is overridden. 
	 * does nothing if handler is not compatible.
	 */
	public void addHandler(String handledClass, String handlerClass) {
		try {
			addHandler(ResourceManager.getInstance().getClass(handledClass), 
					ResourceManager.getInstance().getClass(handlerClass));
		} catch ( Exception ex) {
			System.out.println("error in instantiating handler"+ ex);
		}
	}

	public void addHandler(Class handledClass, Class handlerClass) {
		try {
			Object handler = handlerClass.newInstance();
			if (checkType(handler)) {
				handlers.put(handledClass, handler);
			}
		} catch (InstantiationException e) {
			System.out.println("error in instantiating handler"+ e);
		} catch (IllegalAccessException e) {
			System.out.println("error in instantiating handler"+ e);
		}
		
	}
//----------------------helper for subclasses to implement

	/** checks and returns whether the handler type if compatible with 
	 *  the assumed "similarity" of the collection items.
	 *  Here: always true.
	 */
	protected boolean checkType(Object object) {
		return true;
	}

	protected void initHandlers() {	
	 handlers = createMap();
	 addHandler("java.lang.Object", "java.lang.Object");
	}

//----------------------helper
	
	/** 
	 * PENDING
	 * needs testing: handling of no ancestor mapped correct?
	 * 
	 */
	private Class getUsableValueClass(Class valueClass) {
	  if (valueClass == null) {
	    return null;
	  }
		if (!isMappedDirectly(valueClass)) {
			Class usableInterface = getUsableInterface(valueClass);
			if (usableInterface != null) {
				return usableInterface;
			}
			return getUsableValueClass(valueClass.getSuperclass());
		}
		return valueClass;
	}

	private Class getUsableInterface(Class valueClass) {
		Class[] interfaces = valueClass.getInterfaces();
		for (int i = 0; i < interfaces.length; i++) {
			if (isMappedDirectly(interfaces[i])) {
				return interfaces[i];
			}
		}
		return null;
	}

	private boolean isMappedDirectly(Class valueClass) {
		return handlers.get(valueClass) != null;
	}

//------------------------factory

	protected Map createMap() {
		return new HashMap();
	}


}
