package de.kleopatra.support;

import java.awt.*;
import java.awt.event.MouseListener;

import javax.accessibility.*;
import javax.swing.*;
import javax.swing.text.JTextComponent;

/**	collection of utility methods...
 *
 *
 *	@version $Revision: 1.4 $ $Date: 2004/10/01 13:43:56 $
 * @author (C) Jeanette Winzenburg, Berlin
 */
public class KleoUtilities {

//---------------------------accessible support

	/** recursively walks accessible and returns the accessible with the given name.
	 *  or null if nothing found below.
	 *	POST: resultAccessible.getAccessibleContext().getAccessibleName().equals(accessibleName) OR
	 *		resultAccessible == null
	 *	NOTE: resultAccessible != null implies that getAccessibleContext() != null,
	 *	so clients need not furtheer check for it.
	 */
	public static Accessible findAccessibleByName(Accessible accessible, String accessibleName) {
		if ((accessible == null) || (accessibleName == null)) return null;
		AccessibleContext context = accessible.getAccessibleContext();
		if (context != null) {
			if (accessibleName.equals(context.getAccessibleName())) {
				return accessible;
			}
			for (int i = 0; i < context.getAccessibleChildrenCount(); i++) {
				Accessible result = findAccessibleByName(context.getAccessibleChild(i), accessibleName);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	public static AccessibleAction findAccessibleActionByName(Accessible accessible, String accessibleName) {
		Accessible foundAccessible = findAccessibleByName(accessible, accessibleName);
		if (foundAccessible == null) return null;
		return foundAccessible.getAccessibleContext().getAccessibleAction();
	}

//---------------------------finding child component of certain type

	/** 
	 * returns the first child of type JList below parent.
	 */
	public static JList findListChild(Component parent) {
		if (!(parent instanceof Container)) return null;
		Container cont = (Container) parent;
		if (cont instanceof JList) return (JList) cont;
		for (int i = 0 ; i < cont.getComponentCount() ; i++) {
			JList result = findListChild(cont.getComponent(i));
			if (result != null) {
				return result;
			}
	  }
	  return null;
	}

	/** 
	 * returns the first child of type JTable below parent.
	 */
	public static JTable findTableChild(Component parent) {
		if (!(parent instanceof Container)) return null;
		Container cont = (Container) parent;
		if (cont instanceof JTable ) return (JTable) cont;
		for (int i = 0 ; i < cont.getComponentCount() ; i++) {
			JTable result = findTableChild(cont.getComponent(i));
			if (result != null) {
				return result;
			}
	  }
	  return null;
	}

	/** 
	 * returns the first component with name of type JComponent in parent
	 * (including parent)
	 */
	public static JComponent findJComponentByName(Component parent, String name) {
		if ((parent == null) || (name == null)) return null;
		if (!(parent instanceof JComponent)) return null;
		JComponent cont = (JComponent) parent;
		if (name.equals(parent.getName())) return cont;
		for (int i = 0 ; i < cont.getComponentCount() ; i++) {
			JComponent result = findJComponentByName(cont.getComponent(i), name);
			if (result != null) {
				return result;
			}
	  }
	  return null;

	}

	/** 
	 * returns the first component with name of type JComponent below parent
	 * 
	 */
	public static JComponent findJChildByName(Container parent, String name) {
		for (int i = 0 ; i < parent.getComponentCount() ; i++) {
			JComponent result = findJComponentByName(parent.getComponent(i), name);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JComponent below parent.
	 * if ignoreScrollPane continues search below a JScrollPane with same name.
	 */
	public static JComponent findJChildByName(Container parent, String name, boolean ignoreScrollPane) {
		JComponent child = findJChildByName(parent, name);
		if ((child instanceof JScrollPane) && ignoreScrollPane) {
			child = findJChildByName(child, name);
		}
		return child;
	}	

	/** 
	 * returns the first component with name of type JLabel below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JLabel findLabel(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JLabel) {
			return (JLabel) child;
		}
		return null;
	}
	
	/** 
	 * returns the first component with name of type JTextComponent below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
  public static JTextComponent findTextComponent(JComponent parent, String name) {
    JComponent child = findJChildByName(parent, name, true);
    if (child instanceof JTextComponent) {
      return (JTextComponent) child;
    }
    return null;
  }

	
	/** 
	 * returns the first component with name of type JTextField below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JTextField findTextField(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JTextField) {
			return (JTextField) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JTextField below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JPasswordField findPasswordField(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JPasswordField) {
			return (JPasswordField) child;
		}
		return null;
	}
	
	/** 
	 * returns the first component with name of type JTextArea below parent. 
	 * ignoring JScrollPane. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JTextArea findTextArea(Container parent, String name) {
		JComponent child = findJChildByName(parent, name, true);
		if (child instanceof JTextArea) {
			return (JTextArea) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JFormattedTextField below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JFormattedTextField findFormattedText(JComponent parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JFormattedTextField) {
		  return (JFormattedTextField) child;
		}
    return null;
  }
	
	/** 
	 * returns the first component with name of type JComboBox below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JComboBox findComboBox(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JComboBox) {
			return (JComboBox) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type AbstractButton below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static AbstractButton findAbstractButton(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof AbstractButton) {
			return (AbstractButton) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JRadioButton below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JRadioButton findRadioButton(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JRadioButton) {
			return (JRadioButton) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JCheckBox below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JCheckBox findCheckBox(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JCheckBox) {
			return (JCheckBox) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JSpinner below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JSpinner findSpinner(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JSpinner) {
			return (JSpinner) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JSlider below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JSlider findSlider(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JSlider) {
			return (JSlider) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JProgressBar below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JProgressBar findProgressBar(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JProgressBar) {
			return (JProgressBar) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JScrollBar below parent. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JScrollBar findScrollBar(Container parent, String name) {
		JComponent child = findJChildByName(parent, name);
		if (child instanceof JScrollBar) {
			return (JScrollBar) child;
		}
		return null;
	}
	
	/** 
	 * returns the first component with name of type JList below parent,
	 * ignoring JScrollPane. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
	public static JList findList(Container parent, String name) {
		JComponent child = findJChildByName(parent, name, true);
		if (child instanceof JList) {
			return (JList) child;
		}
		return null;
	}

	/** 
	 * returns the first component with name of type JTable below parent,
	 * ignoring JScrollPane. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
  public static JTable findTable(Container parent, String name) {
    JComponent child = findJChildByName(parent, name, true);
    if (child instanceof JTable) {
      return (JTable) child;
    }
    return null;
  }

	/** 
	 * returns the first component with name of type JTree below parent,
	 * ignoring JScrollPane. 
	 * NOTE: it stops searching if a component with name but wrong type is found.
	 * 
	 */
  public static JTree findTree(Container parent, String name) {
    JComponent child = findJChildByName(parent, name, true);
    if (child instanceof JTree) {
      return (JTree) child;
    }
    return null;
  }
//-------------------------- AWT
  
	/** 
	 * returns the first component with name in parent
	 * (including parent)
	 */
	public static Component findComponentByName(Component parent, String name) {
		if ((parent == null) || (name == null)) return null;
		if (name.equals(parent.getName())) return parent;
		if (!(parent instanceof Container)) return null;
		Container cont = (Container) parent;
		for (int i = 0 ; i < cont.getComponentCount() ; i++) {
			Component result = findComponentByName(cont.getComponent(i), name);
			if (result != null) {
				return result;
			}
	  }
	  return null;

	}


	/** 
	 * returns the first component with name below parent
	 */
	public static Component findChildByName(Container parent, String name) {
		for (int i = 0 ; i < parent.getComponentCount() ; i++) {
			Component result = findComponentByName(parent.getComponent(i), name);
			if (result != null) {
				return result;
			}
		}
		return null;
		
	}

	/** 
	 * returns the first component with name below parent, 
	 * ignoring ScrollPane if ignoreScrollPane is true.
	 */
	public static Component findChildByName(Container parent, String name, boolean ignoreScrollPane) {
		Component child = findJChildByName(parent, name);
		if (((child instanceof JScrollPane) || (child instanceof ScrollPane)) && ignoreScrollPane) {
			child = findChildByName((Container)child, name);
		}
		return child;
	}	

//---------------------------finding listeners of certain classname

	/**
	 * returns a MouseListener with listener.getClass().getName() contains
	 * partOfListenerClassName. Nice for tweaking listener lists.
	 */
	public static MouseListener findMouseListener(Component list, String partOfListenerClassName) {
		if ((list == null) || (partOfListenerClassName == null)) return null;
		MouseListener[] listeners = list.getMouseListeners();
		for (int i = 0 ; i < listeners.length ; i++) {
			String className = listeners[i].getClass().getName();
			if (className.indexOf(partOfListenerClassName) >= 0) {
				return listeners[i];
			}
	  }
		return null;
	}


}
