/*
 * Created on 23.09.2004
 *
 */
package de.kleopatra.view.binding;

import com.jgoodies.binding.value.Trigger;

/**
 * Dump for MetaData required to bind a
 * property to a widget.<p>
 *
 * Typically the MetaData is filled by client code (f.i.
 * the BeanBinding), given to a BindingMap to find an
 * appropriate BindingHandler which in turn is responsible
 * to actually wire the the property to a component. <p>
 *
 * It's up to the BindingMap to decide which handlers to
 * choose. It may use the XXMetaData.class as well as
 * the metaData properties componentClass or targetClass.<p>
 *
 * All setters are meant to be used during the filling
 * phase only <p>
 *
 * NOTE: as all binding api this still has a high
 * probability of change.
 *
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.5 $ - $Date: 2004/10/01 12:27:06 $
 */
public class BindingMetaData {

  protected String componentName;
  protected String propertyName;
  protected Class targetClass;

//  protected Object componentType;
  protected Class componentClass;
  protected boolean commitOnFocusLost;
  protected Trigger focusLostTrigger;

//---------------------- convenience constructors  

  /**
   * PRE: componentName != null.
   * @param componentName
   */
  public BindingMetaData(String componentName) {
    this(componentName, (String) null);
  }

  public BindingMetaData(String componentName, String propertyName) {
    setComponentName(componentName);
    setPropertyName(propertyName);
    initDefaultCommit();
  }

  public BindingMetaData(String componentName, Class componentClass) {
    this(componentName, null, componentClass);
  }

  public BindingMetaData(String componentName, String propertyName,
      Class componentClass) {
    setComponentName(componentName);
    setPropertyName(propertyName);
    setComponentClass(componentClass);
    initDefaultCommit();
  }

  /** returns the name of the component to bind to.
   *
   * @return
   */
  public String getComponentName() {
    return componentName;
  }

  /*
   * PRE: componentName != null.
   */
  public void setComponentName(String componentName) {
    if (componentName == null) {
      throw new NullPointerException("componentName must not be null");
    }
    this.componentName = componentName;
  }

  /** returns the name of the property to be bound.
   *
   * @return the propertyName if explicitly set or the
   *  componentName if propertyName is null.
   */
  public String getPropertyName() {
    return (propertyName != null) ? propertyName : getComponentName();
  }

  /** set the name of the property to be bound.
   *
   * @param propertyName - can be null which implies
   *    propertyName == componentName.
   *
   */
  public void setPropertyName(String propertyName) {
    this.propertyName = propertyName;
  }

  /** returns the class of the bound property.
   *
   * @return
   */
  public Class getTargetClass() {
    return targetClass;
  }

  public void setTargetClass(Class targetClass) {
    this.targetClass = targetClass;
  }

  /** returns the Class of the binding component.
   *
   * @return
   */
  public Class getComponentClass() {
    return componentClass;
  }

  public void setComponentClass(Class componentClass) {
    this.componentClass = componentClass;
  }

  public Trigger getFocusLostTrigger() {
    return focusLostTrigger;
  }

  /** sets the trigger to use on commitOnFocusLost. <p>
   *  side-effect: sets the commitOnFocusLost-property to
   *  trigger != null.
   * @param focusLostTrigger
   */
  public void setFocusLostTrigger(Trigger focusLostTrigger) {
    this.focusLostTrigger = focusLostTrigger;
    setCommitOnFocusLost(focusLostTrigger != null);
  }

  /**
   * returns true if committing the component value to the property
   * should be delayed until a focusLost event.<p>
   *
   * defaults to false.
   *
   * @return
   */
  public boolean isCommitOnFocusLost() {
    return commitOnFocusLost;
  }

  public void setCommitOnFocusLost(boolean commitOnFocusLost) {
    this.commitOnFocusLost = commitOnFocusLost;
  }

//----------------------------------  

  /**
   * hook for subclasses to control default commit behaviour.
   * here: does nothing.
   */
  protected void initDefaultCommit() {}

//  public Object getComponentType() {
//    return componentType;
//  }
//  public void setComponentType(Object componentType) {
//    this.componentType = componentType;
//  }
}
