/*
 * Created on 24.09.2004
 *
 */
package de.kleopatra.view.binding;

import java.util.List;

import javax.swing.JComboBox;
import javax.swing.ListModel;

import com.jgoodies.binding.value.ValueModel;

/**
 * BindingMetaData for use by combobox binding.
 *
 * The property binding is to the selection of the combobox.
 * The list must be provided, either directly as List, ListModel,
 * Object[] or as a ValueModel wired to a list-like property.
 *
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.4 $ - $Date: 2004/10/01 12:27:06 $
 */
public class ComboBoxBindingMetaData extends BindingMetaData {

  private boolean editable;
  private boolean editableSet;
  private Object model;
  public ComboBoxBindingMetaData(String componentName, Object model) {
    super(componentName, JComboBox.class);
    setModel(model);
  }

  public ComboBoxBindingMetaData(String componentName, String propertyName,
      Object model) {
    super(componentName, propertyName, JComboBox.class);
    setModel(model);
  }

  /**
   * sets the model to use for the list part of the combobox.
   * Must be of type List, ListModel, Object[] or ValueModel vending
   * one of those.
   *
   * NOTE: It#s up to client code to guarantee the type returned by the
   * ValueModel - no way to check it here.
   *
   * @param model
   *
   * @throws IllegalArgumentException if type not compatible.
   */
  public void setModel(Object model) {
    checkType(model);
    this.model = model;
  }

  public Object getModel() {
    return model;
  }

  //---------------------- overriding super
  //---------------------- additional accessing 
  public boolean isEditable() {
    return editableSet ? editable : getDefaultEditability();
  }

  public void setEditable(boolean editable) {
    editableSet = true;
    this.editable = editable;
  }

  //-------------------------------- 

  /** check for compatible model types.
   * @throws IllegalArgumentException if not compatible.
   */
  protected void checkType(Object model) {
    if ((model instanceof ListModel) || (model instanceof List)
        || (model instanceof Object[]) || (model instanceof ValueModel)) {
      return;
    }
    throw new IllegalArgumentException(
      "bindable list data data must be Array, ListModel, List, ObservableList. "
      + "\n Alternatively it can be a ValueModel which "
      + "returns one of the above as value");
  }

  protected boolean getDefaultEditability() {
    return false;
  }
}
