/*
 * Created on 30.04.2004
 *
 */
package de.kleopatra.view.binding;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.SwingUtilities;

import com.jgoodies.binding.value.AbstractValueModel;
import com.jgoodies.binding.value.ValueModel;

/**
 *
 * Actually, this is a dirty hack to make sure a listener
 * will be updated with the new value. <p>
 *
 * Need it if the source is not notifying or if a Adapter
 * in the valueModel chain will not accept changes during
 * an update cycle. Should solve more cleanly...
 *
 * <ul>
 * <li> forces a invoked valueChange with the new value only after
 *   setValue here
 * <li> fires a "normal" valueChanged if the source is notifying
 * </ul>
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/07/09 09:13:18 $
 */
public class DelayedRefresher extends AbstractValueModel {

  protected ValueModel source;
  public DelayedRefresher(ValueModel source) {
    this.source = source;
    source.addValueChangeListener(createValueChangeListener());
  }

  private PropertyChangeListener createValueChangeListener() {
    PropertyChangeListener l = new PropertyChangeListener() {

      public void propertyChange(PropertyChangeEvent evt) {
        fireValueChange(evt.getOldValue(), evt.getNewValue());
      }
    };
    return l;
  }

  protected void invokeRefresh(final Object oldValue) {
    SwingUtilities.invokeLater(new Runnable() {

      // force an event
      public void run() {
        fireValueChange(null, getValue());
      }
    });
  }

  public Object getValue() {
    return source.getValue();
  }

  public void setValue(Object newValue) {
    Object oldValue = getValue();
    source.setValue(newValue);
    invokeRefresh(oldValue);
  }
}
