/*
 * Created on 04.05.2004
 *
 */
package de.kleopatra.view.binding;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;

import javax.swing.AbstractButton;
import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.binding.beans.PropertyAdapter;

/**
 * trying to couple a AbstractButton's selected state to enable state
 * of dependents.
 *
 * NOTE: selected is not a bound property of AbstractButton...
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.3 $ - $Date: 2004/10/04 11:56:04 $
 */
public class EnableMultiplexer {

  private PropertyAdapter switchingAdapter;
  private Set dependents;
  private boolean isInvers;
  public EnableMultiplexer(AbstractButton source) {
    initSwitch(source);
  }

  public void addDependent(JComponent comp) {
    getDependents().add(comp);
    update(comp, switchingAdapter.getValue());
  }

  public void setInvers(boolean isInvers) {
    if (this.isInvers == isInvers) {
      return;
    }
    this.isInvers = isInvers;
    multiplex(switchingAdapter.getValue());
  }

//----------------------helper
  private void update(JComponent comp, Object value) {
    comp.setEnabled(getDependentState(value));
  }

  private boolean getDependentState(Object value) {
    if (!(value instanceof Boolean)) {
      throw new IllegalArgumentException("must be Boolean");
    }
    boolean state = ((Boolean) value).booleanValue();
    return isInvers ? !state : state;
  }

  private Set getDependents() {
    if (dependents == null) {
      dependents = createSet();
    }
    return dependents;
  }

  private void multiplex(Object value) {
    if (dependents == null) {
      return;
    }
    for (Iterator iterator = getDependents().iterator();
        iterator.hasNext(); ) {
      JComponent comp = (JComponent) iterator.next();
      update(comp, value);
    }
  }

//---------------------- init  
  private void initSwitch(AbstractButton source) {
    switchingAdapter = new PropertyAdapter(source, "selected", false);
    source.addChangeListener(createChangeListener());
  }

  private ChangeListener createChangeListener() {
    ChangeListener l = new ChangeListener() {

      public void stateChanged(ChangeEvent e) {
        multiplex(switchingAdapter.getValue());
      }
    };
    return l;
  }

  private PropertyChangeListener createSwitchListener() {
    PropertyChangeListener l = new PropertyChangeListener() {

      public void propertyChange(PropertyChangeEvent evt) {
        multiplex(evt.getNewValue());
      }
    };
    return l;
  }

  private Set createSet() {
    return new HashSet();
  }
}
