/*
 * Created on 24.09.2004
 *
 */
package de.kleopatra.view.binding;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JRadioButton;

/**
 * BindingMetaData for use by binding a group of radio options.
 *
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.3 $ - $Date: 2004/10/01 12:27:06 $
 */
public class RadioBindingMetaData extends BindingMetaData {

  private List componentIDs;
  private List optionValues;

//------------- convenience constructors

  /**
   *
   * PRE: lists != null, not empty and of same size
   * @param componentIDs the componentNames of the radioButtons
   * @param propertyName the property to bind
   * @param optionValues  the values bound to the selected state of
   *   the corresponding button
   *
   */
  public RadioBindingMetaData(List componentIDs, String propertyName,
      List optionValues) {
    super(String.valueOf(componentIDs.get(0)).toString(), propertyName,
          JRadioButton.class);
    setLists(componentIDs, optionValues);
  }

  /** set synched lists of componentIDs and optionValues.
   *
   * @param componentIDs the componentNames of the radioButtons
   * @param optionValues  the values bound to the selected state of
   *   the corresponding button
   *
   * @throws IllegalArgumentException if lists don't have
   *  the same size.
   */
  public void setLists(List componentIDs, List optionValues) {
    if (componentIDs.size() != optionValues.size()) {
      throw new IllegalArgumentException("list sizes must be equal");
    }
    this.componentIDs = new ArrayList(componentIDs);
    this.optionValues = new ArrayList(optionValues);
  }

//----------------------- accessing
//  public List getComponentIds() {
//    return componentIDs;
//  }
//  
//  public List getOptionValues() {
//    return optionValues;
//  }
  public String getComponentID(int index) {
    return (String) componentIDs.get(index);
  }

  public Object getOptionValue(int index) {
    return optionValues.get(index);
  }

  public int getGroupSize() {
    return componentIDs.size();
  }
}
