package de.kleopatra.view.factory;

import java.awt.Component;
import java.awt.Container;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import de.kleopatra.view.presentation.DefaultPresentationManager;
import de.kleopatra.view.presentation.PresentationManager;

/**	minimal implementation for testing AWT.
 *
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/05/05 10:02:18 $
 */
public class AWTUIFactory implements UIFactory {

	private ComponentTypeMap componentTypes;
	private PresentationManager presentationManager;
	private final static Logger LOGGER = Logger.getLogger("AWTUIFactory");
	
	public AWTUIFactory() {
		installDefaultComponentTypes();
	}

	public Container createDefaultContainer(Object componentID) {
		return (Container) createComponent(componentID, getDefaultContainerType());
	}

//---------------------------UIFactory - basics

	public Component createComponent(Object componentID, Object componentType) {
		Component result = createAndCustomizeComponent(componentID, componentType);
		if (result != null) {
			return result;
		}
		return createAndCustomizeComponent(componentID, getFallBackComponentType());
	}

	public void setComponentTypeMap(ComponentTypeMap componentTypeMap) {
		this.componentTypes = componentTypeMap;
	}

	public ComponentTypeMap getComponentTypeMap() {
		return componentTypes;
	}

//---------------------------uiFactory: experimental

	public PresentationManager getPresentationManager() {
		if (presentationManager == null) {
			presentationManager = createPresentationManager();
		}
		return presentationManager;
	}

	protected PresentationManager createPresentationManager() {
		return new DefaultPresentationManager();
	}

	/** tries to extract the componentID from a given comp.
	 *	result may be null.
	 *	PRE: comp != null
	 *
	 */
	public Object getComponentID(Component comp) {
		if (comp.getName() != null) {
			return comp.getName();
		}
		return guessComponentID(comp);
	}

	private int count;
	private String defaultName = "component";

//---------------------------specifics

	protected Component createAndCustomizeComponent(Object componentID, Object componentType) {
		if (componentTypes != null) {
			Object comp =  componentTypes.getComponentInstance(componentType);
			if (comp instanceof Component) {
				return customizeComponentByComponentID((Component) comp, componentID);
			}
		} 

		return null;
	}


	public Component customizeComponentByComponentID(Component comp, Object componentID) {
		customizeComponentDefaultProperty(comp, componentID);
	 	comp.setName(String.valueOf(componentID));
	 	return comp;
	}

	protected void customizeComponentDefaultProperty(Component comp, Object componentID) {
		String methodName = "setText";
		if (customizeComponentDefaultProperty(comp, componentID, "setText")) return;
		customizeComponentDefaultProperty(comp, componentID, "setLabel");
	}

	protected boolean customizeComponentDefaultProperty(Component comp, Object componentID, String methodName) {
		boolean success = false;
		Class clazz = comp.getClass();
		Class[] parameters = new Class[] { String.class };
		try {
			Method method = clazz.getMethod(methodName, parameters);
			method.invoke(comp, new Object[] { String.valueOf(componentID) });
			return true;
		} catch (Exception ex) {
			LOGGER.warning("Error in method invoke - can't do anything... "+ ex);
		}
		return false;
	}

	protected Object guessComponentID(Component comp) {
		Class clazz = comp.getClass();
		String methodName = "getText";
		try {
			Method method = clazz.getMethod(methodName, null);
			return method.invoke(comp, null);
		} catch (Exception ex) {
			LOGGER.warning("Error in method invoke - can't do anything... " + ex);
		}
		return null;
	}

//---------------------------helper


	protected void installDefaultComponentTypes() {
		componentTypes = createComponentTypeMap();
		initComponentTypeMap(componentTypes);
	}

	private void initComponentTypeMap(ComponentTypeMap componentMap) {
		componentMap.addComponentType("Label", "java.awt.Label");
		componentMap.addComponentType("TextField", "java.awt.TextField");
		componentMap.addComponentType("Button", "java.awt.Button");
		componentMap.addComponentType("RadioButton", "java.awt.Checkbox");
		componentMap.addComponentType("CheckBox", "java.awt.Checkbox");
		componentMap.addComponentType("ComboBox", "java.awt.Choice");
		componentMap.addComponentType("Panel", "java.awt.Panel");
		componentMap.addComponentType("TextArea", "java.awt.TextArea");
	}

	protected Object getFallBackComponentType() {
		return getDefaultContainerType();
	}

	protected Object getDefaultContainerType() {
		return "Panel";
	}

//---------------------------factory methods - lazy instantiation

//---------------------------factory methods

	protected ComponentTypeMap createComponentTypeMap() {
		return new DefaultComponentTypeMap();
	}

	protected Map createMap() {
		return new HashMap();
	}

}
