/*
 * Created on 05.09.2003
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package de.kleopatra.view.factory;

import java.util.Map;
import java.util.Set;

import javax.swing.*;

/**
 * Type map that's roughly initialized to standard Swing JComponent classes.
 * 
 * <p>
 * Note: subclasses which want to hook into custom factories
 * for the types already registered here should make sure to 
 * remove the entry from the corresponding componentMap.
 * Or subclass DefaultComponentTypeMap.
 * 
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.2 $ - $Date: 2004/09/29 14:39:41 $
 */
public class SwingComponentTypeMap extends DefaultComponentTypeMap {

	protected Map instantiatorMap;

	public SwingComponentTypeMap() {
		initComponentTypes();
		initContainerTypes();
		initScrollableTypes();
		initInstantiatorTypes();
	}

//--------------------------overriding super's public methods
	
	public Set getComponentTypes() {
		Set map =  createSet();
		map.addAll(super.getComponentTypes());
		map.addAll(getInstantiatorMap().keySet());
		return map;
	}
	
//--------------------------overriding super's helper	
	
	protected Object factoryHook(Object componentType) {
		Instantiator instantiator = (Instantiator) getInstantiatorMap().get(componentType);
		if (instantiator != null) {
			return instantiator.instantiate();
		}
		return null;
	}
	
//--------------------------helper

	protected void addComponentInstantiator(Object componentType, Instantiator instantiator) {
		getInstantiatorMap().put(componentType, instantiator);
	}
	
//--------------------------init 

	protected void initScrollableTypes() {
		addScrollableType("List");
		addScrollableType("TextArea");
		addScrollableType("TextPane");
		addScrollableType("EditorPane");
		addScrollableType("Table");
		addScrollableType("Tree");
	}

	protected void initComponentTypes() {
		addComponentType("Label", javax.swing.JLabel.class);
		addComponentType("TextField", javax.swing.JTextField.class);
		addComponentType("TextField.formatted", javax.swing.JFormattedTextField.class);
		addComponentType("TextField.password", javax.swing.JPasswordField.class);
		// custom classes to workaround different bugs 
		addComponentType("Button", de.kleopatra.view.JButton.class);
		addComponentType("RadioButton", de.kleopatra.view.JRadioButton.class);
		addComponentType("CheckBox", de.kleopatra.view.JCheckBox.class);
		addComponentType("ToggleButton", de.kleopatra.view.JToggleButton.class);
		addComponentType("ComboBox", javax.swing.JComboBox.class);
		addComponentType("Separator", de.kleopatra.forms.view.JTitledSeparator.class);
		addComponentType("TextArea", javax.swing.JTextArea.class);
		addComponentType("TextPane", javax.swing.JTextPane.class);
		addComponentType("EditorPane", javax.swing.JEditorPane.class);
		addComponentType("List", javax.swing.JList.class);
		// custom table to have better sizing behaviour
		addComponentType("Table", de.kleopatra.view.table.JFilledTable.class); //*/ "javax.swing.JTable");
		addComponentType("Tree", javax.swing.JTree.class);
		addComponentType("Spinner", javax.swing.JSpinner.class);
		addComponentType("Slider", javax.swing.JSlider.class);
		addComponentType("ScrollBar", javax.swing.JScrollBar.class);
		addComponentType("ProgressBar", javax.swing.JProgressBar.class);
		addComponentType("PlaceHolder", de.kleopatra.view.JTitledPlaceHolder.class);
	}
	
	protected void initContainerTypes() {
		addContainerType("Panel", javax.swing.JPanel.class);
		addContainerType("BorderedPanel", de.kleopatra.view.JTitledBorderPanel.class);
		addContainerType("ScrollPane", javax.swing.JScrollPane.class);
	}

	protected void initInstantiatorTypes() {
		addComponentInstantiator("Separator.vertical", new Instantiator() {
			public Object instantiate() {
				return new JSeparator(SwingConstants.VERTICAL);
			}
		});
		addComponentInstantiator("Slider.vertical", new Instantiator() {

			public Object instantiate() {
				return new JSlider(SwingConstants.VERTICAL);
			}
			
		});
		addComponentInstantiator("ScrollBar.horizontal", new Instantiator() {

			public Object instantiate() {
				return new JScrollBar(SwingConstants.HORIZONTAL);
			}
			
		});
		addComponentInstantiator("ProgressBar.vertical", new Instantiator() {

			public Object instantiate() {
				
				return new JProgressBar(SwingConstants.VERTICAL);
			}
			
		});
		// PENDING: needs more thought
//		addComponentInstantiator("Label.explaining", new Instantiator() {
//
//			public Object instantiate() {
//				JLabel label = new JLabel();
//				label.putClientProperty("swingempire.noLabelEnding", Boolean.TRUE);
//				return label;
//			}
//			
//		});

//		addComponentInstantiator("ButtonBar.okCancel", new Instantiator() {
//
//      public Object instantiate() {
//        JComponent buttonBar = new JPanel();
//        buttonBar.putClientProperty("ButtonBar", "okCancel");
//        return buttonBar;
//      }
//		 
//		  
//		});
//		addComponentInstantiator("ButtonBar.okCancelApply", new Instantiator() {
//
//      public Object instantiate() {
//        JComponent buttonBar = new JPanel();
//        buttonBar.putClientProperty("ButtonBar", "okCancelApply");
//        return buttonBar;
//      }
//		 
//		  
//		});

	}

	public interface Instantiator {
		Object instantiate();
	}

	protected Map getInstantiatorMap() {
		if (instantiatorMap == null) {
			instantiatorMap = createMap();
		}
		return instantiatorMap;
	}


}
