package de.kleopatra.view.presentation;

import java.awt.event.KeyEvent;
import java.util.*;

import javax.swing.*;

import de.kleopatra.persistence.ResourceManager;

/** Convenience to set action values.
 *  Translates Action.XX_KEYs into somthing easier to remember.
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/05/05 10:00:39 $
 */
public class ActionHelper {

//---------------------------convenience for setting/getting action values by name.
	public static void setActionCommand(Action action, String actionCommand) {
		action.putValue(Action.ACTION_COMMAND_KEY, actionCommand);
	}

	public static String getActionCommand(Action action) {
		return (String) action.getValue(Action.ACTION_COMMAND_KEY);
	}

	public static String getName(Action action) {
		return (String) action.getValue(Action.NAME);
	}

	public static void setMnemonic(Action action, char ch) {
		action.putValue(Action.MNEMONIC_KEY, new Integer(ch));
	}

	public static char getMnemonic(Action action) {
		Integer mnem = (Integer) action.getValue(Action.MNEMONIC_KEY);
		if (mnem != null) {
			return (char) mnem.intValue();
		}
		return 0;
	}

	public static void setName(Action action, String actionName) {
		action.putValue(Action.NAME, actionName);
	}

	public static void setToolTipText(Action action, String toolTipText) {
		action.putValue(Action.SHORT_DESCRIPTION, toolTipText);
	}

	public static String getToolTipText(Action action) {
		return (String) action.getValue(Action.SHORT_DESCRIPTION);
	}

	public static void setHelpText(Action action, String helpText) {
		action.putValue(Action.LONG_DESCRIPTION, helpText);
	}

	public static String getHelpText(Action action) {
		return (String) action.getValue(Action.LONG_DESCRIPTION);
	}

	public static Icon getIcon(Action action) {
		return (Icon) action.getValue(Action.SMALL_ICON);
	}

	public static void setIcon(Action action, Icon icon) {
		action.putValue(Action.SMALL_ICON, icon);
	}

	public static void setAccelerator(Action action, String keyStrokeName) {
		setAccelerator(action, KeyStroke.getKeyStroke(keyStrokeName));
	}

	public static void setAccelerator(Action action, KeyStroke keyStroke) {
		action.putValue(Action.ACCELERATOR_KEY, keyStroke);
	}

	public static KeyStroke getAccelerator(Action action) {
		return (KeyStroke) action.getValue(Action.ACCELERATOR_KEY);
	}

//---------------------------convenience for combined customization of single action

	/** customizes: sets actionCommand to name, mnemonic to first letter.
	 *  setting the actionCommand explicitely is a convenience because the AbstractAction
	 *  constructor sets the name instead of the command.
	 *  command is better for uniquely identifying the action - used as ID.
	 */
	public static void customize(Action action, boolean defaultMnemonic) {
		copyNameToCommand(action);
		if (defaultMnemonic) {
			setDefaultMnemonic(action);
		}
	}

	/** copies Action.NAME to Action.ACTIONCOMMAND if command == null.
	 */
	public static void copyNameToCommand(Action action) {
		if (getActionCommand(action) == null) {
			setActionCommand(action, getName(action));
		}
	}

	public static void setDefaultMnemonic(Action action) {
		String name = getName(action);
		if ((name != null) && (name.length() > 0)) {
			setMnemonic(action, Character.toUpperCase(name.charAt(0)));
		}
	}

	public static void setName(Action action, Map actionNames) {
		if (actionNames == null) {
			return;
		}
		String name = (String) actionNames.get(getActionCommand(action));
		if (name != null) {
			setName(action, name);
		}
	}

	public static void setIcon(Action action, Map iconFilenames, Class clazz) {
		if (iconFilenames == null) {
			return;
		}
		String iconFile = (String) iconFilenames.get(getActionCommand(action));
		setIcon(action, getIconResource(iconFile, clazz));
	}

	public static Icon getIconResource(String filename, Class clazz) {
		return (filename == null)
					 ? null : ResourceManager.getInstance().getIcon(filename, clazz);
	}

	public static void setAccelerator(Action action, Map accelerators) {
		if (accelerators == null) {
			return;
		}
		Object accelerator = accelerators.get(getActionCommand(action));
		if (accelerator instanceof String) {
			setAccelerator(action, (String) accelerator);
		} else if (accelerator instanceof KeyStroke) {
			setAccelerator(action, (KeyStroke) accelerator);
		}
	}

	// pending: should rename to setToolTipText!! singular
	public static void setToolTipTexts(Action action, Map toolTips) {
		if (toolTips == null) {
			return;
		}
		String toolTip = (String) toolTips.get(getActionCommand(action));
		if (toolTip != null) {
			setToolTipText(action, toolTip);
		}
	}

	public static void setHelpText(Action action, Map helpTexts) {
		if (helpTexts == null) {
			return;
		}
		String helpText = (String) helpTexts.get(getActionCommand(action));
		if (helpText != null) {
			setHelpText(action, helpText);
		}
	}

//---------------------------convenience of combined setting of bulk actions
// common PRE: actions != null
// common post: delegates to single combined methods if the source map
// does have an entry for the given actionCommand.

	/** copies Action.NAME to Action.ACTION_COMMAND if command == null.
	 *  sets name to value as found in actionNames.
	 *  sets defaultMnemonic to first char of name.
	 */
	public static void customize(List actions, Map actionNames) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			copyNameToCommand(action);
			setName(action, actionNames);

//      setDefaultMnemonic(action);
		}
	}

	public static void setDefaultMnemonics(List actions) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			setDefaultMnemonic(action);
		}
	}

	public static void setNames(List actions, Map actionNames) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			setName((Action) iterator.next(), actionNames);
		}
	}

	public static void setIcons(List actions, Map iconFilenames, Class clazz) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			setIcon(action, iconFilenames, clazz);
		}
	}

	public static void customizeAccelerators(List actions, Map accelerators) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			setAccelerator(action, accelerators);
			customizeToolTip(action);
		}
	}

	public static void setToolTipTexts(List actions, Map toolTips) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			setToolTipTexts(action, toolTips);
		}
	}

	/** sets the tooltip to a combination of tooltip and accelerator. */
	public static void customizeToolTip(Action action) {
		String accelerator = getAcceleratorText(action);
		String toolTip = getToolTipText(action);

		if (accelerator != null) {
			accelerator = " (" + accelerator + ")";
			if ((toolTip != null)	&& (toolTip.indexOf(accelerator) < 0)) {
				toolTip += accelerator;
			} else {
				toolTip = accelerator;
			}	
			setToolTipText(action, toolTip);
		}
	}

	public static void customizeFromPresentations(List actions,
			List presentation, Class clazz) {
		Map presentationMap = getMap(presentation);
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			customizeFromPresentation((Action) iterator.next(), presentationMap,
					clazz);
		}
	}

	public static void customizeFromPresentation(Action action,
			Map presentation, Class clazz) {
		PresentationDescription desc =
			(PresentationDescription) presentation.get(getActionCommand(action));
		if (desc == null) {
			return;
		}
		customizeFromPresentation(action, desc, clazz);
	}

	public static void customizeFromPresentation(Action action,
			PresentationDescription desc, Class clazz) {
			setName(action, desc.getText());
			setAccelerator(action, desc.getAccelerator());
			setToolTipText(action, desc.getToolTipText());
			setIcon(action, getIconResource(desc.getIcon(), clazz));
			setHelpText(action, desc.getHelpText());
		String mnemonic = desc.getMnemonic();
		if ((mnemonic != null) && (mnemonic.length() > 0)) {
			setMnemonic(action, Character.toUpperCase(mnemonic.charAt(0)));
		}
		customizeToolTip(action);
	}

	public static void configureTab(JTabbedPane pane, int tabIndex, Action action) {
		pane.setTitleAt(tabIndex, ActionHelper.getName(action));
		char mnemonic = getMnemonic(action);
		if (mnemonic > 0) {
			pane.setMnemonicAt(tabIndex, mnemonic);
		}
		String toolTip = getToolTipText(action);
		if (toolTip != null) {
			pane.setToolTipTextAt(tabIndex, toolTip);
		}
		Icon icon = getIcon(action);
		if (icon != null) {
			pane.setIconAt(tabIndex, icon);
		}
		
	}
	private static Map getMap(List presentation) {
		Map map = new HashMap();
		for (Iterator iterator = presentation.iterator(); iterator.hasNext(); ) {
			PresentationDescription desc =
				(PresentationDescription) iterator.next();
			map.put(String.valueOf(desc.getPresentationID()), desc);
		}
		return map;
	}

	public static void setHelpTexts(List actions, Map helpTexts) {
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			setHelpText(action, helpTexts);
		}
	}

	private static String getAcceleratorText(Action action) {
		KeyStroke keyStroke = (KeyStroke) action.getValue(Action.ACCELERATOR_KEY);
		if (keyStroke == null) {
			return null;
		}
		String result = "";
		int modifier = keyStroke.getModifiers();
		if (modifier > 0) {
			result += KeyEvent.getKeyModifiersText(modifier);
			result += "-";
		}
		int keyCode = keyStroke.getKeyCode();
		if (keyCode != 0) {
			result += KeyEvent.getKeyText(keyCode);
		} else {
			result += keyStroke.getKeyChar();
		}
		return result;
	}

	public static void registerActions(List actions, JComponent target,
			int inputMapType) {
		ActionMap actionMap = target.getActionMap();
		InputMap inputMap = target.getInputMap(inputMapType);
		for (Iterator iterator = actions.iterator(); iterator.hasNext(); ) {
			Action action = (Action) iterator.next();
			KeyStroke keyStroke = getAccelerator(action);
			String command = getActionCommand(action);
			if ((keyStroke != null) && (command != null)) {
				inputMap.put(keyStroke, command);
				actionMap.put(command, action);
			}
		}
	}
}
