/*
 * Created on 17.08.2004
 *
 */
package de.kleopatra.view.presentation;

import java.net.URL;
import java.util.ResourceBundle;

/**
 * Handler for localization support and synchronization of 
 * a .properties-file and presentation. <p>
 * 
 * Entries in files with format: 
 * <pre>
 * <formID>.<constraintID>.<attributeKey>=<attributeValue>
 * </pre>
 * is mapped to a Presentation with formID:
 * <pre>
 * 	PresentationDescription desc = findDescription(constraintID);
 * 	desc.setAttribute(attributeKey, attributeValue)
 * </pre>
 * 
 * @author (C) 2004 Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/08/19 11:18:33 $
 */
public interface BundleHandler {
  
  /** Find a resourceBundle in the default locale and apply
   *  to all presentation attributes of the given presentation. <p>
   *  
   *  
   * @param presentation
   * @param targetClass
   */
  void localize(Presentation presentation, Class targetClass);
  
  /** Apply the resourceBundle to all presentation
   *  attributes of the presentation. <p>
   * 
   * @param bundle
   * @param presentation
   */
  void updatePresentation(ResourceBundle bundle,
      Presentation presentation);
  
  /** Merge the presentation attributes of the presentation
   *  into the properties found at the file location bundle. <p>
   * 
   * If there the file does not yet exist, it will be created.
   * If the file already exists it should be encoded like Properties
   * (that is ISO 8859-1 with escaped unicode chars). A block
   * representing the attributes should be written (replacing
   * a possible existing block for this presentation), unrelated
   * content is preserved as is.<p> 
   *  
   * @param presentation
   * @param bundle
   */
  void updateBundle(Presentation presentation, URL bundle);
  
}