package de.kleopatra.view.presentation;

import java.beans.*;
import java.util.*;

/** Default implementation.<p>
 * 
 *  Provides a custom PersistenceDelegate which guarantees
 *  to write out extended attributes correctly. 
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.2 $ - $Date: 2004/09/30 10:18:21 $
 *
 */
public class DefaultPresentationDescription
		implements PresentationDescription {

	private static final List COMMON_ATTRIBUTE_KEYS = Collections.unmodifiableList(
		 Arrays.asList(new Object[] { 
		    ACCELERATOR_KEY, TEXT_KEY, HELP_TEXT_KEY, 
		    TOOL_TIP_TEXT_KEY, MNEMONIC_KEY, ICON_KEY 
			}
		 )
	);

  protected Map attributes;
	protected Object presentationID;


	/** pre: presentationID != null).
	 */
	public DefaultPresentationDescription(Object presentationID) {
		setPresentationID(presentationID);
		setText(presentationID.toString());
	}

	/** meant to be immutable.
	 *  so no public setter.
	 */
	public Object getPresentationID() {
		return presentationID;
	}

	public String getText() {
		return getAttribute(TEXT_KEY);
	}

	public String getToolTipText() {
		return getAttribute(TOOL_TIP_TEXT_KEY);
	}

	public String getHelpText() {
		return getAttribute(HELP_TEXT_KEY);
	}

	public String getAccelerator() {
		return getAttribute(ACCELERATOR_KEY);
	}

	public String getMnemonic() {
		return getAttribute(MNEMONIC_KEY);
	}

	public String getIcon() {
		return getAttribute(ICON_KEY);
	}

	public String getAttribute(String attributeKey) {
		return (String) getAttributes().get(attributeKey);
	}

	public void setText(String text) {
		setAttribute(TEXT_KEY, text);
	}

	public void setToolTipText(String text) {
		setAttribute(TOOL_TIP_TEXT_KEY, text);
	}

	public void setHelpText(String text) {
		setAttribute(HELP_TEXT_KEY, text);
	}

	public void setAccelerator(String keyStrokeText) {
		setAttribute(ACCELERATOR_KEY, keyStrokeText);
	}

	public void setMnemonic(String mnemonic) {
		setAttribute(MNEMONIC_KEY, mnemonic);
	}

	public void setIcon(String iconFile) {
		setAttribute(ICON_KEY, iconFile);
	}

	public void setAttribute(String attributeKey, String attributeValue) {
		if ("".equals(attributeValue)) {
			attributeValue = null;
		}
		getAttributes().put(attributeKey, attributeValue);
	}

  public void clearAttributes() {
    attributes = null;
  }

  public List getAttributeKeys() {
    if (attributes == null) return Collections.EMPTY_LIST;
    List list = createList();
    Set keys = attributes.keySet();
    list.addAll(keys);
    return list;
  }
  
	private List createList() {
    return new ArrayList();
  }

  public String toString() {
		return "ID=" + String.valueOf(presentationID) + " "
					 + String.valueOf(attributes);
	}

	public static PersistenceDelegate createPersistenceDelegate() {
		DefaultPersistenceDelegate delegate = new DefaultPersistenceDelegate() {

			protected void initialize(Class clazz, Object oldInstance,
					Object newInstance, Encoder out) {
				super.initialize(clazz, oldInstance, newInstance, out);
				DefaultPresentationDescription description = (DefaultPresentationDescription) oldInstance;
				for (Iterator iterator = description.getAttributes().entrySet().iterator(); iterator.hasNext();) {
          Map.Entry element = (Map.Entry) iterator.next();
					if (!isCommonAttribute(element.getKey())) {
						out.writeStatement(new Statement(oldInstance, "setAttribute",
								new Object[]{ element.getKey(), element.getValue() }));
					}
          
        }
			}

      private boolean isCommonAttribute(Object attributeKey) {
        return COMMON_ATTRIBUTE_KEYS.contains(attributeKey);
      }
		};
		return delegate;
	  
	}
//---------------------------temporary hacks for XMLEncoder/-Decoder - do not use...
	public DefaultPresentationDescription() {}

	public void setPresentationID(Object presentationID) {
		this.presentationID = presentationID;
	}

//	public void setAttributeMap(Map attributeMap) {
//	  this.attributes = attributeMap;
//	}
//	
//	public Map getAttributeMap() {
//	  return getAttributes();
//	}
//---------------------------factory
	protected Map getAttributes() {
		if (attributes == null) {
			attributes = createMap();
		}
		return attributes;
	}

	protected Map createMap() {
		return new HashMap();
	}


}
