package de.kleopatra.view.presentation;

import java.awt.Component;
import java.awt.Container;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.border.Border;

import de.kleopatra.forms.view.JTitledSeparator;
import de.kleopatra.persistence.ResourceManager;
import de.kleopatra.support.KleoUtilities;
import de.kleopatra.view.JTitledPlaceHolder;

/** PresentationManager for Swing.
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.4 $ - $Date: 2004/10/04 12:18:34 $
 *
 */
public class DefaultPresentationManager implements PresentationManager {

	protected CustomizerMap customizerMap;
	
	public DefaultPresentationManager() {
		initCustomizers();
	}


//---------------------------PresentationMaanger

	public void customize(Container container, Presentation presentation) {
		customize(container, presentation, null);
	}

	public void customize(Container container, Presentation presentation, Class targetClass){
		// HACK for temporary state
		tempResourceID = presentation.getPresentationID();
		tempActionFactory = customActionFactory;
		for (Iterator iterator =
				presentation.getPresentationDescriptions().iterator(); iterator
					.hasNext(); ) {
			PresentationDescription desc =
				(PresentationDescription) iterator.next();
			Component comp = findComponent(container, desc.getPresentationID());
			if (comp != null) {
				getCustomizer(comp).customize(comp, desc, targetClass);
				Object labelID = getProbableLabelID(desc.getPresentationID());
				Component label = findComponent(container, labelID);
				if (label != null) {
					ComponentCustomizer customizer = getCustomizer(label);
					customizer.customize(label, desc, targetClass);
					if (customizer instanceof LabelCustomizer) {
						((LabelCustomizer) customizer).setLabelFor(label, comp);
					}
				}
			}
		}
		customizeContainer(container, presentation, targetClass);
		// HACK for temporary state
		tempResourceID = null;
		tempActionFactory = null;
	}

	public PresentationDescription createPresentationDescription(Component comp) {
		return getCustomizer(comp).createDescription(comp);
	}

	public CustomizerMap getCustomizerMap() {
		return customizerMap;
	}

	public void setCustomizerMap(CustomizerMap map) {
		this.customizerMap = map;
	}

	/** Set the actionFactory to use if different from the shared one.
	 * 	NOTE: used at design-time only (to decouple actions in 
	 *  preview from application scope actions).
	 */
	public void setActionFactory(ActionFactory actionFactory) {
		this.customActionFactory = actionFactory;
	}
	
//----------------------hacking around...

	// hack: quick fix to make ButtonCustomizer static
	protected static Object tempResourceID;

	/** arrggg...  static to access from static  buttonCustomizer. */
	private ActionFactory customActionFactory;
	/** this is valid during the lifetime of customize only. */
	private static ActionFactory tempActionFactory;

	/** returns current ActionFactory.
	 *  HACK: valid during lifetime of customization of container only.
	 * @return actionFactory to register actions.
	 */
	public static ActionFactory getActionFactory() {
		if (tempActionFactory == null) return ActionFactory.getInstance();
		return tempActionFactory;
	}
	
	/** returns current presentation/groupID.
	 *  HACK: valid during lifetime of customization of container only.
	 * @return presentationID of currently customized container.
	 */
	public static Object getGroupID() {
		return tempResourceID;
	}

//---------------------------helper
	protected Object getProbableLabelID(Object id) {
		if (id == null) {
			return null;
		}
		return id.toString() + LABEL_POSTFIX;
	}

	/** removed from public interface.
	 * 
	 */
	protected ComponentCustomizer getCustomizer(Component comp) {
		return customizerMap.getCustomizer(comp.getClass());
	}


	protected Component findComponent(Container container, Object id) {
		if (id == null) {
			return null;
		}
		return KleoUtilities.findChildByName(container, id.toString(), true);
	}

	/** customize the parent form itself if  there is a presentationDescription with
	 *  the name and no child with the same name.
	 * 
	 * @param container
	 * @param presentation
	 * @param targetClass
	 */
	private void customizeContainer(Container container,
			Presentation presentation, Class targetClass) {
		PresentationDescription desc = findDescription(presentation,
																		 container.getName());
		if ((desc != null)
				&& (findComponent(container, container.getName()) == null)) {
			customizerMap.getFormCustomizer().customize(container, desc, targetClass);
		}
	}

	private PresentationDescription findDescription(Presentation presentation,
			Object id) {
		if (id == null) {
			return null;
		}
		for (Iterator iterator =
				presentation.getPresentationDescriptions().iterator(); iterator
					.hasNext(); ) {
			PresentationDescription element =
				(PresentationDescription) iterator.next();
			if (id.equals(element.getPresentationID())) {
				return element;
			}
		}
		return null;
	}

//---------------------------static classes for customizers
	//---------------------------default

	/**
	 *   Default: sets tooltip.
	 * 
	 *	<p>
	 * @version $Revision: 1.4 $ - $Date: 2004/10/04 12:18:34 $
	 */
	public static class DefaultComponentCustomizer
			implements ComponentCustomizer {

		public void customize(Component comp, PresentationDescription desc) {
			customize(comp, desc, null);
		}

		public void customize(Component comp, PresentationDescription desc,
				Class clazz) {
			if (!(comp instanceof JComponent)) {
				return;
			}
			((JComponent) comp).setToolTipText(desc.getToolTipText());
		}

		public PresentationDescription createDescription(Component comp) {
			PresentationDescription desc =
				createPresentationDescription(comp.getName());
			desc.setText(comp.getName());
			if (comp instanceof JComponent) {
				desc.setToolTipText(((JComponent) comp).getToolTipText());
			}
			return desc;
		}

		protected PresentationDescription createPresentationDescription(
				String name) {
			return new DefaultPresentationDescription(name);
		}
	}

//-----------------------------parent container

	/**
	 *   .
	 *	<p>
	 * @version $Revision: 1.4 $ - $Date: 2004/10/04 12:18:34 $
	 */
	public static class PanelCustomizer extends DefaultComponentCustomizer {

		public void customize(Component comp, PresentationDescription desc,
				Class clazz) {
			if (!(comp instanceof JPanel)) {
				return;
			}
			customizePanel((JPanel) comp, desc);
		}

		private void customizePanel(JPanel panel, PresentationDescription desc) {
			if (shouldDecorate(panel, desc)) {
				decorateBoder(panel, desc.getText());
			}
		}

		private boolean shouldDecorate(JPanel panel,
				PresentationDescription desc) {
			return (desc.getText() != null) && (panel.getBorder() != null)
						 && (desc.getText().length() > 0);
		}

		private void decorateBoder(JPanel panel, String title) {
			Border border = panel.getBorder();
			Border titled = BorderFactory.createTitledBorder(title);
			panel.setBorder(BorderFactory.createCompoundBorder(titled, border));
		}
	}

	//---------------------------button

	/**
	 * Customizer for AbstractButtons.
	 * <p>
	 * if the button's action is null  
	 * let the current ActionFactory create a default action with the
	 * button's id as actionCommand and register the action on the factories
	 * list with the current group id (that's why we need the hacky 
	 * static access to the PresentationManager's current id and factory) 
	 * and set the action to the button.
	 * <p>
	 * customize the action with the presentation attributes: 
	 * name, accelerator (? buttons not listening), tooltip, icon, long description,
	 * mnemonic.
	 * 
	 * 
	 *   
	 */
	public static class ButtonCustomizer extends DefaultComponentCustomizer {

		public void customize(Component comp, PresentationDescription desc,
				Class clazz) {
			if (!(comp instanceof AbstractButton)) {
				super.customize(comp, desc, clazz);
				return;
			}
			customizeButton((AbstractButton) comp, desc, clazz);
		}

		public void customizeButton(AbstractButton button,
				PresentationDescription desc, Class clazz) {
			Action action = button.getAction();
			if (action == null) {
				action = createDefaultAction(button.getName());
				button.setAction(action);
			}
			ActionHelper.customizeFromPresentation(action, desc, clazz);
		}

		//---------------------------factory methods
		protected Action createDefaultAction(String actionCommand) {
			return getActionFactory().registerAction(actionCommand,
					getGroupID());
		}
	}

	//---------------------------label

	/**
	 *   Customizer for JLabel.
	 * <p>
	 * customize toolTip, text, icon, mnemonic from presentation attributes.
	 * supports enhanced labelFor (listens to enable/disable of target).<p>
	 * 
	 * Prepared to append colon if useLabelEnding(..) returns true. 
	 * The feature is disabled by default (because it requires some
	 * additional mechanism to decide when not to append - see examples
	 * for one possibility to do it). 
	 * 
	 * 
	 */
	public static class LabelCustomizer extends DefaultComponentCustomizer {
	  private String labelEnding = ":";
	  
		public void customize(Component comp, PresentationDescription desc,
				Class clazz) {
			super.customize(comp, desc, clazz);
			if (!(comp instanceof JLabel)) {
				return;
			}
			customizeLabel((JLabel) comp, desc, clazz);
		}

		public void customizeLabel(JLabel label, PresentationDescription desc,
				Class clazz) {
			label.setIcon((desc.getIcon() == null)
										? null
										: ResourceManager.getInstance().getIcon(desc.getIcon(),
											clazz));
			String mnemonic = desc.getMnemonic();
			if ((mnemonic != null) && (mnemonic.length() > 0)) {
				label.setDisplayedMnemonic(Character.toUpperCase(mnemonic.charAt(0)));
			}
		  setLabelText(label, desc.getText());
		}

	/** set label text, appending colon if appropriate.
	 * 
	 * @param label
	 * @param text
	 */	
	protected void setLabelText(JLabel label, String text) {
      if (useLabelEnding(label, text)) {
        text += getLabelEnding();
      }
      label.setText(text);
    }

	/** 
	 * Decision about appending colon, based on label and text. 
	 * Here: returns false.
	 *  
	 * @param label
	 * @param text
	 * @return
	 */
  protected boolean useLabelEnding(JLabel label, String text) {
    return false;
//    return !Boolean.TRUE.equals(label.getClientProperty(UIProperties.NO_LABELENDING)) &&
//        useLabelEnding(text);
  }

  /** Decision about appending colon, based on text.
   * 
   * @param text
   * @return
   */
  protected boolean useLabelEnding(String text) {
    return (text != null) && !text.endsWith(getLabelEnding());
  }

  protected String getLabelEnding() {
    return labelEnding;
  }

  /** set target as labelFor of JLabel and installs listener to 
	 *  enable/disable label in synch with target.
	 */
		public void setLabelFor(Component label, Component target) {
			if (!(label instanceof JLabel)) {
				return;
			}
			JLabel labelFrom = (JLabel) label;
			labelFrom.setLabelFor(target);
			PropertyChangeListener l = createPropertyChangeListener(labelFrom);
			target.addPropertyChangeListener(l);
		}

		private PropertyChangeListener createPropertyChangeListener(
				final Component dependent) {
			PropertyChangeListener l = new PropertyChangeListener() {

				public void propertyChange(PropertyChangeEvent evt) {
					if ("enabled".equals(evt.getPropertyName())) {
						dependent
							.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
					}
				}
			};
			return l;
		}
	}

//---------------------------separator

	/**
	 *   set tooltip, text of JTitledSeparator.
	 */
	public static class SeparatorCustomizer extends DefaultComponentCustomizer {

		public void customize(Component comp, PresentationDescription desc,
				Class clazz) {
			super.customize(comp, desc, clazz);
			if (!(comp instanceof JTitledSeparator)) {
				return;
			}
			((JTitledSeparator) comp).setText(desc.getText());
		}
	}

//---------------------------separator


	/**
	 *   set tooltip, text of JTitledPlaceHolder.
	 */
	public static class PlaceHolderCustomizer extends DefaultComponentCustomizer {

		public void customize(Component comp, PresentationDescription desc,
				Class clazz) {
			super.customize(comp, desc, clazz);
			if (!(comp instanceof JTitledPlaceHolder)) {
				return;
			}
			((JTitledPlaceHolder) comp).setText(desc.getText());
		}
	}


//---------------------------init
	protected void initCustomizers() {
		customizerMap = createCustomizerMap();
	}

	protected CustomizerMap createCustomizerMap() {
		return new SwingCustomizerMap();
	}

//---------------------------factory methods
}
