package de.kleopatra.view.presentation;

import java.util.List;

/** Represent set of general presentation attributes. <p>
 * 
 *  PresentationID is id to link all kinds of descriptions (layout, binding, ...)
 *  Common attributes are: text, tooltip, accelerator, help text, tooltip text,
 *  mnemonic, small icon. They are accessible via dedicated getters/setters.
 *  <p>
 *  Extended attributes are handled via key/value pairs 
 *  in get/setPresentationAttribute.
 *  <p>
 * 
 *  NOTE: attribute keys can be added on request.<p>
 * 
 * 	CHANGED: the string values of the keys are changed
 * 	to be the same as the properties they represent.
 *  Please let me know if that's a problem in older
 *  resources (should not, because the property is saved).<p>
 * 
 *  
 * 
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2004/08/19 11:18:34 $
 *
 */
public interface PresentationDescription {
//
//  /** common attribute: key for accelerator. */
//	public static final String ACCELERATOR_KEY = "AcceleratorKey";
//  /** common attribute: key for text/label. */
//	public static final String NAME_KEY = "Name";
//  /** common attribute: key for help text. */
//	public static final String HELP_KEY = "LongDescription";
//  /** common attribute: key for tooltip. */
//	public static final String TOOL_TIP_KEY = "ShortDescription";
//  /** common attribute: key for mnemonic. */
//	public static final String MNEMONIC_KEY = "Mnemonic";
//  /** common attribute: key for small icon. */
//	public static final String ICON_KEY = "SmallIcon";

  /** common attribute: key for accelerator. */
	public static final String ACCELERATOR_KEY = "accelerator";
  /** common attribute: key for text/label. 
   *  NOTE: the constant name formerly was NAME_TEXT.
   */
	public static final String TEXT_KEY = "text";
  /** common attribute: key for help text. */
	public static final String HELP_TEXT_KEY = "helpText";
  /** common attribute: key for tooltip. */
	public static final String TOOL_TIP_TEXT_KEY = "toolTipText";
  /** common attribute: key for mnemonic. */
	public static final String MNEMONIC_KEY = "mnemonic";
  /** common attribute: key for small icon. */
	public static final String ICON_KEY = "icon";
	
	
	/** extended attribute: key for formatting mask. 
	 * @deprecated: use MASK_FORMAT_KEY instead.
	 */
	public static final String MASK_KEY = "Mask";

	/** extended attribute: key for formatting mask. 
	 */
	public static final String FORMAT_MASK_KEY = "formatMask";
	/** meant to be immutable.
	 *  so no public setter.
	 */
	public Object getPresentationID();

	/** need for editing...
	 */
	public void setPresentationID(Object id);

	public String getText();

	public String getToolTipText();

	public String getHelpText();

	public String getAccelerator();

	public String getMnemonic();

	public String getIcon();

	public String getAttribute(String attributeKey);

	public void setText(String text);

	public void setToolTipText(String text);

	public void setHelpText(String text);

	public void setAccelerator(String keyStrokeText);

	public void setMnemonic(String mnemonic);

	public void setIcon(String iconFile);

	public void setAttribute(String attributeKey, String attributeValue);

  public void clearAttributes();

  public List getAttributeKeys();
}
