package de.kleopatra.view.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.EventObject;

import javax.swing.*;
import javax.swing.border.LineBorder;

/**	Fixed JTable.GenericCellEditor.
 *	<g>
 *	Changes
 *	tries to base the type of the created object on the type of the given value first
 *  instead of on the type as returned by table.getColumnClass as the original does.
 *	the latter is doing nonsense if the columnClass is an interface
 *
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2003/10/14 11:13:47 $
 *
 */
public class FixedGenericCellEditor extends DefaultCellEditor {

	Class[] argTypes = new Class[]{ String.class };
	java.lang.reflect.Constructor constructor;
	Object value;
	public FixedGenericCellEditor() {
		super(new JTextField());
	}

	public boolean isCellEditable(EventObject e) {
		boolean result = super.isCellEditable(e);
		if (result && (e instanceof KeyEvent)) {
			result = !((InputEvent) e).isConsumed() && 
			// reject accelerators
			// PENDING: not working because JTable does not route the event
			// in processKeyBinding when trying to start edits
				((InputEvent) e).getModifiers() == 0;
		}
		return result;
	}

	public boolean stopCellEditing() {
		String s = (String) super.getCellEditorValue();

		// Here we are dealing with the case where a user
		// has deleted the string value in a cell, possibly
		// after a failed validation. Return null, so that
		// they have the option to replace the value with
		// null or use escape to restore the original.
		// For Strings, return "" for backward compatibility.
		if ("".equals(s)) {
			if (constructor.getDeclaringClass() == String.class) {
				value = s;
			}
			super.stopCellEditing();
		}
		try {
			value = constructor.newInstance(new Object[]{ s });
		} catch (Exception e) {
			((JComponent) getComponent()).setBorder(new LineBorder(Color.red));
			return false;
		}
		return super.stopCellEditing();
	}

	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		this.value = null;
		((JComponent) getComponent()).setBorder(new LineBorder(Color.black));

		try {
			Class type = table.getColumnClass(column);
			if (value != null) {
				type = value.getClass();
			}
			// Since our obligation is to produce a value which is
			// assignable for the required type it is OK to use the
			// String constructor for columns which are declared
			// to contain Objects. A String is an Object.
			if ((type == Object.class) || type.isInterface()) {
				type = String.class;
			}
			constructor = type.getConstructor(argTypes);
		} catch (Exception e) {
			return null;
		}
		return super.getTableCellEditorComponent(table, value, isSelected, row,
				column);
	}

	public Object getCellEditorValue() {
		return value;
	}
}
