package de.kleopatra.view.table;

import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;
import javax.swing.event.MouseInputListener;
import javax.swing.table.JTableHeader;

/** updates the table's selection on mouseEvents.
 *
 *	adjusted c&p from BasicTableUI
 *
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2003/10/14 11:13:47 $
 *
 *
 */
public class HeaderSelection {

	protected JTableHeader header;
	protected MouseInputListener headerListener;
	public HeaderSelection() {

		// TODO: Add your code here
	}

	public void setTableHeader(JTableHeader header) {
		JTableHeader oldHeader = this.header;
		if (oldHeader != null) {
			oldHeader.removeMouseListener(getHeaderListener());
			oldHeader.removeMouseMotionListener(getHeaderListener());
		}
		this.header = header;
		if (header != null) {
			header.addMouseListener(getHeaderListener());
			header.addMouseMotionListener(getHeaderListener());
		}
	}

	//------------------------inner class
	public class HeaderListener extends MouseInputAdapter {

		private boolean selectedOnPress;
		public void mousePressed(MouseEvent e) {
			if (e.isConsumed()) {
				selectedOnPress = false;
				return;
			}
			selectedOnPress = true;
			adjustFocusAndSelection(e);
		}

		public void mouseReleased(MouseEvent e) {
			if (selectedOnPress) {
				if (shouldIgnore(e)) {
					return;
				}
				setValueIsAdjusting(false);
			} else {
				adjustFocusAndSelection(e);
			}
		}

		public void mouseDragged(MouseEvent e) {
			if (shouldIgnore(e)) {
				return;
			}
			Point p = e.getPoint();
			int row =
				header.getTable().getSelectionModel().getAnchorSelectionIndex();
			int column = header.columnAtPoint(p);

			// The autoscroller can generate drag events outside the Table's range.
			if ((column == -1) || (row == -1)) {
				return;
			}
			header.getTable().changeSelection(row, column, false, true);
		}

		private void adjustFocusAndSelection(MouseEvent e) {
			if (shouldIgnore(e)) {
				return;
			}
			Point p = e.getPoint();
			int row =
				header.getTable().getSelectionModel().getAnchorSelectionIndex();
			int column = header.columnAtPoint(p);

			// The autoscroller can generate drag events outside the Table's range.
			if ((column == -1) || (row == -1)) {
				return;
			}
			boolean adjusting = (e.getID() == MouseEvent.MOUSE_PRESSED)
														? true : false;
			setValueIsAdjusting(adjusting);
			header.getTable().changeSelection(row, column, e.isControlDown(),
					e.isShiftDown());
		}

		private boolean shouldIgnore(MouseEvent e) {
			return e.isConsumed()
						 || (!(SwingUtilities.isLeftMouseButton(e)
									 && header.isEnabled()));
		}

		private void setValueIsAdjusting(boolean flag) {
			header.getTable().getSelectionModel().setValueIsAdjusting(flag);
			header.getTable().getColumnModel().getSelectionModel()
				.setValueIsAdjusting(flag);
		}
	}    // end inner class

	//---------------------------factory - lazy
	protected MouseInputListener getHeaderListener() {
		if (headerListener == null) {
			headerListener = createMouseListener();
		}
		return headerListener;
	}

	//---------------------------factory
	protected MouseInputListener createMouseListener() {
		return new HeaderListener();
	}
}
