package de.kleopatra.view.table;


import java.awt.Color;
import java.awt.Component;

import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;


/**	enable tooltips for renderers.
 *	PENDING: handle selection as well
 *
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2003/10/14 11:13:47 $
 *
 */
public class SelectableRenderer implements TableCellRenderer {
	protected TableCellRenderer delegate;
	protected boolean showSelection;
	protected Color originalBackground;
	protected Color selectedBackground;

	public SelectableRenderer(TableCellRenderer delegate, boolean show) {
		this.delegate = delegate;
		setSelectionShowing(show);
	}

	public void setSelectionShowing(boolean showSelection) {
		this.showSelection = showSelection;
	}

	public boolean isSelectionShowing() {
		return showSelection;
	}


	public Component getTableCellRendererComponent(JTable table, Object value,
		boolean isSelected, boolean hasFocus, int row, int col) {
		if (originalBackground == null) {
			initBackground(table, value, row, col);
		}
		Component comp = delegate.getTableCellRendererComponent(table, value,
			isSelected, hasFocus, row, col);
		if ((table != null) && (row == -1)) {
			isSelected = table.isColumnSelected(col);
		}
		setSelectionBackground(comp, isSelected);
		return comp;
	}

//---------------------------helper

	protected void setSelectionBackground(Component comp, boolean isSelected) {
		if (isSelected) {
			comp.setBackground(selectedBackground);
		} else {
			comp.setBackground(originalBackground);
		}
	}

	protected void initBackground(JTable table, Object value, int row, int col) {
		Component comp = delegate.getTableCellRendererComponent(table, value,
			false, false, row, col);
		originalBackground = comp.getBackground();
		// PENDING: need something better - looks terrible in LFs with 
		// originalbackground == null (like WinXP)
		if (originalBackground != null) {
			selectedBackground = originalBackground.darker();
		}
	}

}
