package de.kleopatra.view.util;

import java.awt.Frame;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/** Convenience for simple file choosing with filters.
 *  at the moment: supports only one filter	based on extension
 *
 * @author (C) Jeanette Winzenburg, Berlin
 * @version $Revision: 1.1.1.1 $ - $Date: 2003/10/14 11:13:47 $
 *
 */
public class FileChooserHandler extends FileFilter {

	private JFileChooser chooser;
	private Frame owner;
	private String extension;
	private File startDirectory;
	private Map extensionMap;

	public FileChooserHandler() {
	}

	public FileChooserHandler(JFileChooser chooser) {
		setFileChooser(chooser);
	}

//---------------------------accessing

	public void setFileChooser(JFileChooser chooser) {
		this.chooser = chooser;
		initStartDirectory();
		chooser.setCurrentDirectory(startDirectory);
	}

	public void setStartDirectory(File startDirectory) {
		this.startDirectory = startDirectory;
		if (chooser != null) {
			chooser.setCurrentDirectory(startDirectory);
		}
	}

	public void setOwner(Frame owner) {
		this.owner = owner;
	}

	public void registerExtension(String ext, String description) {
		ext = ext.toLowerCase();
		getExtensionMap().put(ext, buildDescription(ext, description));
	}

	public String getFilename(int option, String extension) {
		this.extension = extension;
		JFileChooser chooser = getFileChooser();
		// some bug: has to be done before setting the filter...
		chooser.setSelectedFile(getCorrectedExtension(chooser.getSelectedFile()));
		chooser.setAcceptAllFileFilterUsed(true);
		chooser.setFileFilter(this);
		chooser.setDialogType(option);
		int action = chooser.showDialog(owner, null);
		if (action == JFileChooser.APPROVE_OPTION) {
			return chooser.getSelectedFile().getPath();
		}
		return null;
	}

	public String ensureExtension(String filename, String extension) {
		filename = filename.trim();
		int lastPoint = getExtensionPosition(filename);
		if (lastPoint > 0) {
//			return extension.toLowerCase().equals(filename.substring(lastPoint + 1, filename.length()).toLowerCase());
			filename = filename.substring(0, lastPoint);
		}
		return filename + extension;
	}


//---------------------------FileFilter

	public boolean accept(File file) {
		if (file.isDirectory()) return true;
		return hasValidExtension(file.getName());
	}

	public String getDescription() {
		String description = (String) getExtensionMap().get(extension);
		return description == null ? extension : description;
	}

//---------------------------helper

	protected boolean hasValidExtension(String filename) {
		filename = filename.trim();
		int lastPoint = getExtensionPosition(filename);
		if (lastPoint > 0) {
			return extension.equalsIgnoreCase(filename.substring(lastPoint, filename.length()));
		}
		return false;
	}

	protected int getExtensionPosition(String filename) {
		return filename.lastIndexOf(".");
	}

	protected File getCorrectedExtension(File file) {
		if ((file == null) || (accept(file))) return file;
		String corrected = ensureExtension(file.getPath(), extension);
		return new File(corrected);
	}

	protected String buildDescription(String ext, String basicDescription) {
		return basicDescription + " (" + ext + ")";
	}

	protected Map getExtensionMap() {
		if (extensionMap == null) {
			extensionMap = createMap();
		}
		return extensionMap;
	}

	protected JFileChooser getFileChooser() {
		if (chooser == null) {
			setFileChooser(createFileChooser());
		}
		return chooser;
	}

//---------------------------init

	protected void initStartDirectory() {
		String prop = System.getProperty("user.dir");
		startDirectory = new File(prop);
	}

//---------------------------factory methods

	protected Map createMap() {
		return new HashMap();
	}

	protected JFileChooser createFileChooser() {
		return new JFileChooser();
	}
}
